%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating cointegration          %
%          relations under linear restrictions simultaneously    %
%          with linear restrictions on alpha and Psi             %
%                                                                %
%          Function: alpha_beta_restrict                         %
%                                                                %
%          Input:                                                %
%                       Z0: nxT matrix with endogenous variables %
%                       Z1: nq1xT matrix with lagged variables   %
%                           in levels                            %
%                       Z2: mxT matrix with remaining regressors %
%                       Z3: m+rxT matrix with regressors for VEC %
%                       newbeta: nq1xr matrix with restricted    %
%                           ML estimates of the cointegration    %
%                           vectors                              %
%                       Theta: nxk matrix with unrestricted      %
%                           estimates of short-run parameters    %
%                       Rmat: matrix with restrictions on        %
%                           vec(theta)                           %
%                       rvec: vector with values for             %
%                           restrictions                         %
%                       dcoint: indicator if constant or trend   %
%                           in the cointegration space           %
%                       q1: number of exogneous I(1) variables   %
%                       modelsdir: text string with path to the  %
%                           H_i.rest files with restrictions     %
%                           (can be a string with a space)       %
%                       modelfile: name of model                 %
%                       showwaitbar: integer (1=show)            %
%                                                                %
%          Output:                                               %
%                       theta: nxk matrix with restricted        %
%                          estimates of short-run parameters     %
%                       Sigma_theta: estimated asymptotic        %
%                          covariance matrix of vec(theta)       %
%          	            newbeta: nq1xr matrix with restricted    %
%          	               cointegration vectors                 %
%                       newstdbeta: nq1xr matrix with estimated  %
%                          asymptotic conditional                %
%                          standard errors                       %
%                       Z3: m+rxT matrix with regressors for VEC %
%                       infomat: 1x2 vector with number of       %
%                          iterations and convergence value      %
%                       abort: text string indicating if the     %
%                          cancel button was pressed or not      %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [theta,Sigma_theta,newbeta,newstdbeta,Z3,infomat,abort] = alpha_beta_restrict(Z0,Z1,Z2,Z3,newbeta,Theta,Rmat,rvec,dcoint,q1,modelsdir,modelfile,showwaitbar)

if nargin<13;
   showwaitbar = 0;
end;

n = size(Theta,1);
r = size(newbeta,2);
T = size(Z0,2);
newstdbeta = zeros(size(newbeta));

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);
%
% starting up the estimation by getting a restricted estimate of alpha and Psi
%
[theta,Sigma_theta,infomat,abort] = linrest(Z0,Z3,Theta,Rmat,rvec,0,1);
g2 = size(theta,2);
alphar = theta(:,g2-r+1:g2);
if g2>r;
   Psir = theta(:,1:g2-r);
else;
   Psir = [];
end;
neweps = Z0-(theta*Z3);
newlnl = loglike(neweps);
newOmega = (1/T)*neweps*neweps';
sivec = zeros(r,1);
%
% bH is needed to check the gradient of the likelihood function
% wrt the beta parameters
%
for i=1:r;
   file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
   if fileexist(file)==1;
      h = loaddata(file);
      if i==1;
         H = h;
         bH = h;
      else;
         H = [H h];
         bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
      end;
      sivec(i,1) = size(h,2);
   end;
end;
%
% Get the normalizing variables
%
file = strcat(modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.norm');
if fileexist(file)==1;
   normvec = loaddata(file);
else;
   normvec = [];
end;

%
% Setting up the normalizing restrictions
%

if isempty(normvec)==0;
   nq1 = size(newbeta,1);
   addrestmat = zeros(nq1*r,r);
   for i=1:r;
      thisnorm = normvec(i,1);
      addrestmat(thisnorm+nq1*(i-1),i) = 1;
   end;
else;
   addrestmat = [];
end;

%
% constructing restriction matrices for beta
%

Knr = commutation(n+q1+dcoint,r);
bH_ort = null(bH','r');
bH_ort = [bH_ort addrestmat];
bRmat = bH_ort'*Knr';
if isempty(addrestmat)==1;
   brvec = zeros(size(bRmat,1),1);
else;
   brvec = [zeros(size(bRmat,1)-r,1);ones(r,1)];
end;
bH = null(bH_ort','r');

%
% Computing the gradient using the new theta matrix
%

M_01 = (1/T)*Z0*Z1';
M_11 = (1/T)*Z1*Z1';
if isempty(Z2)==0;
   M_21 = (1/T)*Z2*Z1';
   M_12 = M_21';
   M_22 = (1/T)*Z2*Z2';
else;
   M_21 = [];
   M_12 = [];
   M_22 = [];
end;
if rank(newOmega,eps)<size(newOmega,1);
   inv_newOmega = eye(n);
else;
   inv_newOmega = newOmega\eye(n);
end;
if isempty(M_21)==0;
   grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)-(Psir*M_21)))*bH;
else;
   grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)))*bH;
end;
grad = max(abs(grad));

%
% setting up iterations if needed
%
if grad>ccrit;
   %
   % setting up progress dialogue
   %
   if showwaitbar==1;
      input_var.title = ['Progress for maximum of ' int2str(curr_ini.maxit) ' iterations'];
      input_var.facecolor = curr_ini.progress_facecolor;
      input_var.startfacecolor = curr_ini.progress_startfacecolor;
      input_var.edgecolor = curr_ini.progress_edgecolor;
      input_var.bgcolor = curr_ini.progress_bgcolor;
      input_var.stop = 1;
      input_var.clock = 1;
      input_var.name = 'ML Estimation of Alpha,Beta under Restrictions';
      mywait = progressdlg(0,input_var);
%      set(mywait,'Name','ML Estimation of Alpha,Beta under Restrictions');
      set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
      drawnow;
   end;
   it = 1;
   while (it<=curr_ini.maxit)&(grad>ccrit);
      oldgrad = grad;
      if showwaitbar==1;
         abort = get(mywait,'UserData');
         if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
            break;
         else;
            progressdlg([it/curr_ini.maxit grad]);
         end;
      end;
      oldlnl = newlnl;
      %
      % create new R0 and R1 series using Psir, alphar, and newbeta
      %
      if isempty(Z2)==0;
         newR0 = Z0-(Psir+(alphar*newbeta'*(M_12/M_22)))*Z2;
         newR1 = Z1-(M_12/M_22)*Z2;
      else;
         newR0 = Z0;
         newR1 = Z1;
      end;
      %
      % Constructing unrestricted beta estimate
      %
      S_00_new = (1/T)*newR0*newR0';
      S_10_new = (1/T)*newR1*newR0';
      S_11_new = (1/T)*newR1*newR1';
      S_01_new = (1/T)*newR0*newR1';
      alpharort = null(alphar');
      alpharbar = alphar/(alphar'*alphar);
      S_11_alpha = S_11_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_01_new)));
      S_10_alpha = S_10_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_00_new)));
      S_00_alpha = S_00_new-(S_00_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_00_new)));
      newbeta = (S_11_alpha)\(S_10_alpha*alpharbar);
      %
      % Computing restricted newbeta estimate using R0tilde and R1tilde
      %
      R0tilde = alpharbar'*(eye(n)-(S_00_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'))))*newR0;
      R1tilde = newR1-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\alpharort'))*newR0;
      [thisbeta,sig_beta,infmat,newabort] = linrest(R0tilde,R1tilde,newbeta',bRmat,brvec,0,1);
      newbeta = thisbeta';
      if isempty(Z2)==0;
         Z3 = [Z2;newbeta'*Z1];
      else;
         Z3 = newbeta'*Z1;
      end;
      [theta,Sigma_theta,infomat,newabort,inv_Omega] = linrest(Z0,Z3,theta,Rmat,rvec,0,1);
      g2 = size(theta,2);
      alphar = theta(:,g2-r+1:g2);
      if g2>r;
         Psir = theta(:,1:g2-r);
      else;
         Psir = [];
      end;
      neweps = Z0-(theta*Z3);
      newlnl = loglike(neweps);
      newOmega = (1/T)*neweps*neweps';
      if curr_ini.betagrad==0;
         grad = abs((newlnl-oldlnl)/oldlnl);
         oldgrad = 0;
      else;
         if isempty(M_21)==0;
            oldgrad = T*(vec(alphar))'*kron(eye(r),inv_Omega*(M_01-(alphar*newbeta'*M_11)-(Psir*M_21)))*bH;
         else;
            oldgrad = T*(vec(alphar))'*kron(eye(r),inv_Omega*(M_01-(alphar*newbeta'*M_11)))*bH;
         end;
         oldgrad = max(abs(oldgrad));
         if oldgrad>min([ccrit*10 1.0e-006]);
            oldgrad = 0;
         end;
         if rank(newOmega,eps)<size(newOmega,1);
            inv_newOmega = eye(n);
         else;
            inv_newOmega = newOmega\eye(n);
         end;
         if isempty(M_21)==0;
            grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)-(Psir*M_21)))*bH;
         else;
            grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(M_01-(alphar*newbeta'*M_11)))*bH;
         end;
      end;
      grad = abs(max(abs(grad))-oldgrad);
      infomat = [it grad];
      it = it+1;
   end;
   %
   % close progress dialogue
   %
   if showwaitbar==1;
      set(mywait,'UserData','done');
      close(mywait);
      drawnow;
   end;
   %
   % retrieve the alpha and Psi estimates (needed for conditional
   % standard errors for beta)
   %
   alphar = theta(:,g2-r+1:g2);
   if g2>r;
      Psir = theta(:,1:g2-r);
   else;
      Psir = [];
   end;
   %
   % recompute ths conditional standard errors for beta
   %
   for i=1:r;
      %
      % Selecting the restrictions from H
      %
      si = sivec(i,1);
      if si>=2;
         sj = size(H,2);
         if i==1;
            Hsel = [eye(si);zeros(sj-si,si)];
         elseif i==r;
            Hsel = [zeros(sj-si,si);eye(si)];
         else;
            sk = sivec(1,1);
            for j=2:i-1;
               sk = sk+sivec(j,1);
            end;
            Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
         end;
         Hi = H*Hsel;
         %
         % Determining the normalizations
         %
         normelem = normvec(i,1);
         myexit = 0;
         for j=1:si;
            %
            % Fix Hi to make sure that the normalizing row
            % contains one unit element and zeros
            %
            normmatrix = Hi(normelem,:);
            for k=1:(n+q1+dcoint);
               if k~=normelem;
                  checkHi = [normmatrix;Hi(k,:)];
                  if rank(checkHi*checkHi')==size(checkHi,1);
                     normmatrix = checkHi;
                     if size(normmatrix,1)==size(Hi,2);
                        break;
                     end;
                  end;
               end;
            end;
            Hi = Hi/normmatrix;
            check = Hi(normelem,j);
            if (abs(check)>0)&(check~=1);
               Hi(:,j) = (1/Hi(normelem,j))*Hi(:,j);
               check = Hi(normelem,j);
            end;          
            if check==1;
               %
               % Split Hi into hi and H_i
               %
               if j==1;
                  hi = Hi(:,1);
                  H_i = Hi(:,2:si);
               elseif j==si;
                  hi = Hi(:,si);
                  H_i = Hi(:,1:si-1);
               else;
                  hi = Hi(:,j);
                  H_i = [Hi(:,1:j-1) Hi(:,j+1:si)];
               end;
               %
               % Check H_i to make sure it is of full rank
               %
               check2 = H_i'*H_i;
               if abs(det(check2))>=.000000001;
                  myexit=1;
               end;
            end;
            if myexit==1;
               break;
            end;
         end;
         %
         % Computing the asymptotic covariance for the
         % si-1 free parameters of column i in betar
         %
         information = alphar(:,i)'*inv_newOmega*alphar(:,i)*H_i'*S_11_alpha*H_i;
         covbeta = (1/T)*(H_i/information)*H_i';
         %
         % Computing the asyptotic standard errors and
         % including them in stdbeta
         %
         newstdbeta(:,i) = sqrt(abs(diag(covbeta)));
      end;
   end;
end;

%
% end of alpha_beta_restrict.m
%
