%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing Chow forecast,          %
%          sample-split, and break-point tests recursively       %
%                                                                %
%          Function: chowft                                      %
%                                                                %
%          Input:                                                %
%                        Z0: nxT matrix                          %
%                        Z1: n+dcoint+q1xT matrix                %
%                        Z2: qxT matrix (may be empty            %
%                        beta: n+dcoint+q1xr matrix              %
%                        d: number of unrestricted deterministic %
%                            variables                           %
%                        dcoint: number of restricted            %
%                            deterministic variables             %
%                        T0: relative time period for starting   %
%                            the recursions                      %
%                        samplebeg: first period (real)          %
%                        sampleend: last period (real)           %
%                        freq: data frequence string (y,q,m)     %
%                        waitbar: indicator if a progress        %
%                            dialog should be displayed or not   %
%                            (default=0, no progress dialog)     %
%                        modelfile: name of model                %
%                            (default is empty string)           %
%                        modelsdir: directory where restrictions %
%                            files are stored                    %
%                            (default is empty string            %
%                        userest: whether to estimate under      %
%                            linear restrictions or not          %
%                            (default=0, no restrictions)        %
%                        use_c_rest: whether to estimate under   %
%                            linear restrictions on C            %
%                            (default=0, no restrictions)        %
%                        mm: structure with model information    %
%                                                                %
%          Output:                                               %
%                        my_fmat: T-T1xn matrix of F-tests       %
%                        lrmat: T-T1xn matrix of LR tests        %
%                        dff: T-T1x2 matrix with deg. freedom    %
%                            for F-tests                         %
%                        dflr: T-T1x1 vector with deg. freedom   %
%                            for LR-tests                        %
%                        fpmat: T-T1xn matrix with p-values for  %
%                            F-tests (if m-file available)       %
%                        lrpmat: T-T1xn matrix with p-values for %
%                            LR-tests (if m-file available)      %
%                        dstr: text string providing the         %
%                            sample dates used in the recursions %
%                        abort: text string indicating if        %
%                            the iterations have been cancelled  %
%                            or not                              %
%                        chow_ss: vector with sample-split tests %
%                        chow_bp: vector with break-point tests  %
%                        df_ss: vector with degrees of freedom   %
%                            for sample split tests              %
%                        df_bp: matrix with degrees of freedom   %
%                            for break-point tests               %
%                        chow_dates: string matrix with break    %
%                            point dates                         %
%                        from_to_vec: vector with observation    %
%                            numbers used in the sample-split    %
%                            and break-point tests               %
%                        chow_ss_pval: vector with p-values for  %
%                            sample-split tests                  %
%                        chow_bp_pval: vector with p-values for  %
%                            break-point tests                   %
%                        my_epsmat: matrix with 1-step ahead     %
%                            forecast errors                     %
%                                                                %
%          NOTE: requires that Z2 and beta are not both empty    %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [my_fmat,lrmat,dff,dflr,fpmat,lrpmat,dstr,abort,chow_ss,chow_bp,df_ss,df_bp,chow_dates,from_to_vec,chow_ss_pval,chow_bp_pval,my_epsmat] = chowft(Z0,Z1,Z2,beta,d,dcoint,T0,samplebeg,sampleend,freq,waitbar,modelfile,modelsdir,userest,use_c_rest,mm)

if nargin<11;
   waitbar = 0;
end;

if nargin<14;
   modelfile = '';
   modelsdir = '';
   userest = 0;
end;

if nargin<15;
   use_c_rest = 0;
   mm = [];
end;

%
% Preliminaries
%

[n T] = size(Z0);

if T0<0;
   T1 = 0;
elseif T0>0.8;
   T1 = round(1*T/2);
else;
   T1 = round(T*T0);
end;

if isempty(beta)==0;
   r = size(beta,2);
   Z3 = [Z2;beta'*Z1];
else;
   r = 0;
   Z3 = Z2;
end;

k = size(Z3,1);

if T1<=size(Z1,1)+size(Z2,1)+(2*n);
   T1 = size(Z1,1)+size(Z2,1)+1+(2*n);
end;
last_T1 = T-(size(Z1,1)+size(Z2,1)+1+(2*n));

%
% Initialize
%

my_fmat = [];
lrmat = [];
dff = [];
dflr = [];
opened = 0;
dstr = '';
chow_ss = [];
chow_bp = [];
df_ss = [];
df_bp = [];
chow_dates = '';
from_to_vec = [];
chow_ss_pval = [];
chow_bp_pval = [];

abort = '';

if T1>T-1;
   fpmat = [];
   lrpmat = [];
   return;
end;

if waitbar==1;
   curr_ini = find_inisettings;
   realscrsz = get(0,'ScreenSize');
   input_var.title = ['Progress for ' int2str(T-T1) ' recursive estimates'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.label = 'Recursion number:';
   input_var.stop = 0;
   input_var.clock = 0;
   input_var.name = 'Recursive Chow Tests';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','Recursive Chow Tests');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;

%
% Full sample estimates
%

b = (Z0*Z3')/(Z3*Z3');

%
% Specify restrictions if asked for (r>0 by construction when userest=1)
%

if userest==1;
   file = strcat(modelsdir,'G-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat,rvec] = rmatrix(G,n,r);
   dfa = size(G,1);
   if isempty(Z2)==0;
      Rmat = [zeros(dfa,size(Z2,1)*n) Rmat];
   end;
   [b,Sigma_b,infomat,dabort] = linrest(Z0,Z3,b,Rmat,rvec,0);
end;
if use_c_rest==1;
   file = strcat(modelsdir,'C-',int2str(n),'x',int2str(r),'-',modelfile,'.rest');
   G = loaddata(file);
   [Rmat_C,rvec_C] = rmatrix(G,n,n);
   [b,Sigma_b,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,dabort] = c_restrict(Z0,Z1,Z2,mm.restrict,mm.beta,b,Rmat_C,rvec_C,mm.p,mm.pe,mm.d,mm.dcoint,mm.q0,mm.q1,mm.modelsdir,mm.modelfile,0,0);
end;

epsilon = Z0-b*Z3;
OmegaT = epsilon*epsilon';
epsmat = [];
Omega = (1/T)*OmegaT;
epsilon_T = epsilon;

%
% Subsample estimates
%

t = T-1;
while t>=T1;
   if waitbar==1;
      if opened==0;
         drawnow;
         progressdlg([(1/(T-T1)) 1]);
         opened = 1;
         start = T1;
      else;
         abort = get(mywait,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            progressdlg([(opened/(T-start)) opened]);
         end;
      end;
   end;
%   T2 = T-t;
   T2 = 1;
   Z0t = Z0(:,1:t);
   Z3t = Z3(:,1:t);
   Z3T1_1 = Z3(:,t+1);
   %
   % Remove any affinely dependent variables from Z3t
   %
   delvar = [];
   if d>0;
      x = Z2(1:d,1:t)';
      if dcoint==1;
         x_ = Z1(size(Z1,1),1:t)';
      else;
         x_ = [];
      end;
      [delvar,keepvar] = getdelvars(x,x_);
      x = checkdeter(x,x_);
      Z3t = [x';Z3t(d+1:k,:)];
      %
      % need to make sure that the correct deterministic
      % variables are chosen (not fixed yet)
      %
      Z3T1_1 = [Z2(keepvar,t+1);Z3T1_1(d+1:k,1)];
      Z2t = [x';Z2(d+1:size(Z2,1),1:t)];
      my_d = size(x',1);
   else;
      if isempty(Z2)==0;
         Z2t = Z2(:,1:t);
      else;
         Z2t = [];
      end;
      my_d = 0;
   end;
   %
   % Estimate time t b coefficients
   %
   b = Z0t*Z3t'/(Z3t*Z3t');
   if userest==1;
      %
      % Fix Rmat if needed
      %
      Rmatt = Rmat;
      if isempty(delvar)==0;
         d1 = size(delvar,1);
         for i=1:d1;
            dk = delvar(i,1);
            rs = size(Rmatt,2);
            if dk==1;
               Rmatt = Rmatt(:,n+1:rs);
            elseif dk==size(b,2);
               Rmatt = Rmatt(:,1:rs-n);
            else;
               Rmatt = [Rmatt(:,1:(dk-1)*n) Rmatt(:,dk*n+1:rs)];
            end;
         end;
      end;
      %
      % Estimate under linear restrictions if asked for
      %
      [b,Sigma_b,infomat,dabort] = linrest(Z0t,Z3t,b,Rmatt,rvec,0);
   end;
   if use_c_rest==1;
      [b,Sigma_b,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,dabort] = c_restrict(Z0(:,1:t),Z1(:,1:t),Z2t,mm.restrict,mm.beta,b,Rmat_C,rvec_C,mm.p,mm.pe,my_d,mm.dcoint,mm.q0,mm.q1,mm.modelsdir,mm.modelfile,0,0);
   end;
   epsilon = Z0t-b*Z3t;
   epsT1_1 = Z0(:,t+1)-b*Z3T1_1;
   epsmat = [epsmat epsT1_1];
   Omegat = epsilon*epsilon';
   %
   % Compute tests
   %
   for i=1:n;
      F = ((t-k)/T2)*(abs((OmegaT(i,i)-Omegat(i,i)))/(Omegat(i,i)));
      LR = T*abs(log(OmegaT(i,i))-log(Omegat(i,i)));
      if i==1;
         fmati = F;
         lrmati = LR;
      else;
         fmati = [fmati F];
         lrmati = [lrmati LR];
      end;
   end;
   my_fmat = [fmati;my_fmat];
   lrmat = [lrmati;lrmat];
   dff = [[T2 t-k];dff];
   dflr = [T2;dflr];
   dstr = strvcat(getdate(samplebeg,sampleend,freq,t/T),dstr);
   OmegaT = Omegat;
   if t<last_T1;
      %
      % now we can deal with sample-split and break-point tests
      %
      first_T = t;
      n_parm_first = n*size(b,2);
      last_T = T-t;
      Omega_first = (1/first_T)*Omegat;
      %
      % create second sample data
      %
      Z0T = Z0(:,t+1:T);
      Z3T = Z3(:,t+1:T);
      %
      % Remove any affinely dependent variables from Z3T
      %
      delvar = [];
      if d>0;
         x = Z2(1:d,t+1:T)';
         if dcoint==1;
            x_ = Z1(size(Z1,1),t+1:T)';
         else;
            x_ = [];
         end;
         [delvar,keepvar] = getdelvars(x,x_);
         x = checkdeter(x,x_);
         Z3T = [x';Z3T(d+1:k,:)];
         Z2T = [x';Z2(d+1:size(Z2,1),t+1:T)];
         my_d = size(x',1);
      else;
         if isempty(Z2)==0;
            Z2T = Z2(:,t+1:T);
         else;
            Z2T = [];
         end;
         my_d = 0;
      end;
      %
      % Estimate the second b coefficients
      %
      bT = Z0T*Z3T'/(Z3T*Z3T');
      if userest==1;
         %
         % Fix Rmat if needed
         %
         RmatT = Rmat;
         if isempty(delvar)==0;
            d1 = size(delvar,1);
            for i=1:d1;
               dk = delvar(i,1);
               rs = size(RmatT,2);
               if dk==1;
                  RmatT = RmatT(:,n+1:rs);
               elseif dk==size(b,2);
                  RmatT = RmatT(:,1:rs-n);
               else;
                  RmatT = [RmatT(:,1:(dk-1)*n) RmatT(:,dk*n+1:rs)];
               end;
            end;
         end;
         %
         % Estimate under linear restrictions if asked for
         %
         [bT,Sigma_bT,infomat,dabort] = linrest(Z0T,Z3T,bT,RmatT,rvec,0);
      end;
      if use_c_rest==1;
         [bT,Sigma_bT,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,dabort] = c_restrict(Z0T,Z1(:,t+1:T),Z2T,mm.restrict,mm.beta,bT,Rmat_C,rvec_C,mm.p,mm.pe,my_d,mm.dcoint,mm.q0,mm.q1,mm.modelsdir,mm.modelfile,0,0);
      end;
      n_parm_last = n*size(bT,2);
      epsilonT = Z0T-bT*Z3T;
      Omega_last = (1/last_T)*epsilonT*epsilonT';
      %
      % compute sample-split test
      %
      Omega_combined = ((1/first_T)*epsilon_T(:,1:t)*epsilon_T(:,1:t)') + ((1/last_T)*epsilon_T(:,t+1:T)*epsilon_T(:,t+1:T)');
      ch_ss = ((first_T+last_T)*log(det(Omega_combined)))-( first_T*log(det(Omega_first)) )-( last_T*log(det(Omega_last)) );
      df_ssT = n_parm_first+n_parm_last-(n*k);
      %
      % compute break-point test
      %
      R_2r = 1-( ((first_T/T)^n)*(det(Omega_first)/det(Omega)) );
      N = T-k-df_ssT-(n-df_ssT+1/2);
      q = ((n*df_ssT)/2)+1;
      s = sqrt( ((n*n*df_ssT*df_ssT) - 4 )/( (n*n)+(df_ssT*df_ssT)-5 ) );
      if round((N*s)-q)>0;
         df_bpT = [n*df_ssT round((N*s)-q)];
         help_bp = (1-R_2r)^(1/s);
         ch_bp = ((1-help_bp)/help_bp)*(df_bpT(1,2)/df_bpT(1,1));
      else;
         ch_bp = 0;
         df_bpT = [1 1];
      end;
      %
      % store results
      %
      chow_dates = strvcat(getdate(samplebeg,sampleend,freq,(t+1)/T),chow_dates);
      chow_ss = [ch_ss;chow_ss];
      chow_bp = [ch_bp;chow_bp];
      df_ss = [df_ssT;df_ss];
      df_bp = [df_bpT;df_bp];
      from_to_vec = [t+1 from_to_vec];
   end;
   opened = opened+1;
   t = t-1;
end;

fpmat = 1-fdis_cdf(my_fmat,dff(:,1)*ones(1,n),dff(:,2)*ones(1,n));
lrpmat = 1-chis_cdf(lrmat,dflr*ones(1,n));

if isempty(chow_ss)==0;
   chow_ss_pval = 1-chis_cdf(chow_ss,df_ss);
   chow_bp_pval = 1-fdis_cdf(chow_bp,df_bp(:,1),df_bp(:,2));
end;

if waitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

if nargout==17;
   my_epsmat = epsmat;
end;

%
% end of chowft.m
%
