%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                              %
%  Function: make_toolbar                                                      %
%                                                                              %
%  Purpose: Create a toolbar with push and toggle buttons for                  %
%           a figure window                                                    %
%                                                                              %
%  Input:                                                                      %
%        aStruct:     a structure (may be empty)                               %
%        par:         parent window (handle)                                   %
%        fldnames:    names for new fields to add to aStruct (cell array)      %
%                     1x(number of buttons+1)                                  %
%        buttonsdata: structure with 2 fields "cdata" and "userdata".          %
%                     The structure is a vector, i.e. buttonsdata(1).cdata,    %
%                     buttonsdata(1).userdata, buttonsdata(2).cdata, etc.      %
%        buttons:     structure with the following fields and entries:         %
%           type:          push or toggle button (cell array)                  %
%           tooltipstr:    cell array with strings for tooltips                %
%           callbackstr:   cell array with callback strings                    %
%           enablestr:     cell array with on/off entries                      %
%           separatorstr:  cell array with on/off entries                      %
%           statestr:      cell array with on/off entries (only used for       %
%                          toggle buttons)                                     %
%           tagstr:        cell array with tag strings                         %
%                                                                              %
%           NOTE: All these fields must have the same number of entries        %
%                 which is equal to the number of buttons                      %
%                                                                              %
%  Output:                                                                     %
%        aStruct:    a structure containing all handles of the created         %
%                    objects+any handles passed to the function                %
%                                                                              %
%  This version:     July 21, 2006                                             %
%  Written by:       Anders Warne                                              %
%                    Copyright (c) 2001-2006                                   %
%                                                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function aStruct = make_toolbar(aStruct,par,fldnames,buttonsdata,buttons)

origStruct = aStruct;

if (ishandle(par)==0)|(strcmp(get(par,'Type'),'figure')==0);
   par = get(0,'CurrentWindow');
end;

h_tool = uitoolbar('Parent',par, ...
         'HandleVisibility','off', ...
         'Interruptible','on', ...
         'Clipping','on');

if (isempty(aStruct)==1)|(isstruct(aStruct)==0);
   aStruct = struct(fldnames{1},h_tool);
else;
   aStruct = setfield(aStruct,fldnames{1},h_tool);
end;

for i=1:length(buttons.type);
   if strcmp(buttons.type{i},'push')==1;
      h_btn = uipushtool('Parent',h_tool, ...
                         'TooltipString',buttons.tooltipstr{i}, ...
                         'ClickedCallback',buttons.callbackstr{i}, ...
                         'CData',buttonsdata(i).cdata, ...
                         'Separator',buttons.separatorstr{i}, ...
                         'Interruptible','on', ...
                         'UserData',buttonsdata(i).userdata, ...
                         'Clipping','on', ...
                         'Enable',buttons.enablestr{i}, ...
                         'Tag',buttons.tagstr{i});
   elseif strcmp(buttons.type{i},'toggle')==1;
      h_btn = uitoggletool('Parent',h_tool, ...
                         'TooltipString',buttons.tooltipstr{i}, ...
                         'ClickedCallback',buttons.callbackstr{i}, ...
                         'CData',buttonsdata(i).cdata, ...
                         'Separator',buttons.separatorstr{i}, ...
                         'State',buttons.statestr{i}, ...
                         'Interruptible','on', ...
                         'UserData',buttonsdata(i).userdata, ...
                         'Clipping','on', ...
                         'Enable',buttons.enablestr{i}, ...
                         'Tag',buttons.tagstr{i});
   else;
      error('You may only use "push" or "toggle" for button type.');
      aStruct = origStruct;;
      return;
   end;
   aStruct = setfield(aStruct,fldnames{1+i},h_btn);
end;

%
% end of make_toolbar.m
%
