%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          Computes multivariate LM tests for autoregressive     %
%          conditional heteroskedasticity                        %
%                                                                %
%          Function: march                                       %
%                                                                %
%          Input:                                                %
%                       epsilon: nxT matrix with residuals       %
%                       t: degrees of freedom from original      %
%                           estimates                            %
%                       rest_march: option if march should       %
%                           estimate restricted MARCH model      %
%                       sscorrect: boolean about F-type          %
%                           correction of test                   %
%                       R_mat: restrictions that MARCH model     %
%                           should support (optional)            %
%                       r_vec: values of the restrictions        %
%                           (optional)                           %
%                                                                %
%          Output:                                               %
%          		          results: test result (value, p-value)    %
%                       df: matrix with degrees of freedom       %
%                                                                %
%          This version:        April 7, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [results,df] = march(epsilon,t,rest_march,sscorrect,R_mat,r_vec)



%
% Preliminaries
%

[n T] = size(epsilon);
if t<=(n*(n+1))/2;
   %
   % there are too few degrees of freedom
   %
   results = [];
   df = [];
   return;
end;
m = (n*(n+1))/2;
%
if nargin==4;
   if rest_march==1;
      %
      % set up restrictions to ensure that b is dialgonal
      %
      R_mat_orth = zeros(m,m*m);
      my_eye = eye(m);
      for i=1:m;
         e_i = my_eye(i,:);
         R_mat_orth(i,:) = kron(e_i,e_i);
      end;
      R_mat = null(R_mat_orth)';
      R_mat = [zeros(size(R_mat,1),m) R_mat];
      r_vec = zeros(m*(m-1),1);
   else;
      R_mat = [];
      r_vec = [];
   end;
end;
%
heps = zeros(m,T);
for i=1:T;
   heps(:,i) = vech(epsilon(:,i)*epsilon(:,i)');
end;
y = heps(:,2:T);
x = [ones(1,T-1);heps(:,1:T-1)];
%
% regress y on x without any restrictions
%
b = y*x'*inverse(x*x');
u = y-(b*x);
Sig_0 = (1/(T-1))*y*y';
if rest_march==1;
   %
   % estimate b under the restrictions
   %
   [b,Sig_b,infomat,abort] = linrest(y,x,b,R_mat,r_vec,0,0,0);
   u = y-(b*x);
   df = m;
else;
   df = m*m;
end;

Sig_1 = (1/(T-1))*u*u';
R_sq = 1-((1/m)*sum(diag(Sig_1*inverse(Sig_0))));

results = T*m*R_sq;

if sscorrect==0;
   pval = 1-chis_cdf(results,df);
   results = [results pval];
else;
%   d_f = ceil(df/m);
%   results = (t/(df*T))*results;
   results = results/df;
   df = [df t];
   pval = 1-fdis_cdf(results,df(1,1),df(1,2));
   results = [results pval];
end;

%
% end of march.m
%
