%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating cointegration          %
%          relations under linear restrictions simultaneously    %
%          with linear restrictions on alpha and (while Psi      %
%          is fixed at the full sample estimate                  %
%                                                                %
%          Function: rec_alpha_beta_restrict                     %
%                                                                %
%          Input:                                                %
%                       Z0: nxT matrix with endogenous variables %
%                       Z1: nq1xT matrix with lagged variables   %
%                           in levels                            %
%                       Z2: mxT matrix with remaining regressors %
%                       Z3: m+rxT matrix with regressors for VEC %
%                       newbeta: nq1xr matrix with restricted    %
%                           ML estimates of the cointegration    %
%                           vectors                              %
%                       Theta: nxk matrix with unrestricted      %
%                           estimates of short-run parameters    %
%                       Rmat: matrix with restrictions on        %
%                           vec(theta)                           %
%                       rvec: vector with values for             %
%                           restrictions                         %
%                       dcoint: indicator if constant or trend   %
%                           in the cointegration space           %
%                       q1: number of exogneous I(1) variables   %
%                       modelsdir: text string with path to the  %
%                           H_i.rest files with restrictions     %
%                           (can be a string with a space)       %
%                       modelfile: name of model                 %
%                                                                %
%          Output:                                               %
%                       theta: nxk matrix with restricted        %
%                          estimates of short-run parameters     %
%                       Sigma_theta: estimated asymptotic        %
%                          covariance matrix of vec(theta)       %
%          	            newbeta: nq1xr matrix with restricted    %
%          	               cointegration vectors                 %
%                       newstdbeta: nq1xr matrix with estimated  %
%                          asymptotic conditional                %
%                          standard errors                       %
%                       Z3: m+rxT matrix with regressors for VEC %
%                                                                %
%          This version:        April 21, 2006                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [alphar,Sigma_alpha,newbeta,newstdbeta,newR0,newR1] = rec_alpha_beta_restrict(newR0,newR1,newbeta,alphar,Rmat,rvec,dcoint,q1,modelsdir,modelfile,Sigma_alpha)

[n r] = size(alphar);
T = size(newR0,2);
newstdbeta = zeros(size(newbeta));

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
ccrit = mystr2num(ch_convcrit);

neweps = newR0-(alphar*newbeta'*newR1);
newlnl = loglike(neweps);
newOmega = (1/T)*neweps*neweps';
sivec = zeros(r,1);

%
% bH is needed to check the gradient of the likelihood function
% wrt the beta parameters
%

for i=1:r;
   file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
   if fileexist(file)==1;
      h = loaddata(file);
      if i==1;
         H = h;
         bH = h;
      else;
         H = [H h];
         bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
      end;
      sivec(i,1) = size(h,2);
   end;
end;
%
% Get the normalizing variables
%
file = strcat(modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.norm');
if fileexist(file)==1;
   normvec = loaddata(file);
else;
   normvec = [];
end;

%
% Setting up the normalizing restrictions
%

if isempty(normvec)==0;
   nq1 = size(newbeta,1);
   addrestmat = zeros(nq1*r,r);
   for i=1:r;
      thisnorm = normvec(i,1);
      addrestmat(thisnorm+nq1*(i-1),i) = 1;
   end;
else;
   addrestmat = [];
end;

%
% constructing restriction matrices for beta
%

Knr = commutation(n+q1+dcoint,r);
bH_ort = null(bH','r');
bH_ort = [bH_ort addrestmat];
bRmat = bH_ort'*Knr';
if isempty(addrestmat)==1;
   brvec = zeros(size(bRmat,1),1);
else;
   brvec = [zeros(size(bRmat,1)-r,1);ones(r,1)];
end;
bH = null(bH_ort','r');

S_00_new = (1/T)*newR0*newR0';
S_10_new = (1/T)*newR1*newR0';
S_11_new = (1/T)*newR1*newR1';
S_01_new = (1/T)*newR0*newR1';
if det(newOmega)<1.0e-014;
   inv_newOmega = 100000*newOmega;
   inv_newOmega = inv_newOmega\eye(n);
   inv_newOmega = 100000*inv_newOmega;
else;
   inv_newOmega = newOmega\eye(n);
end;
grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(S_01_new-(alphar*newbeta'*S_11_new)))*bH;
grad = max(abs(grad));

%
% setting up iterations if needed
%

if grad>ccrit;
   it = 1;
   while (it<=curr_ini.maxit)&(grad>ccrit);
      oldgrad = grad;
      oldlnl = newlnl;
      alpharort = null(alphar');
      alpharbar = alphar/(alphar'*alphar);
      S_11_alpha = S_11_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_01_new)));
      S_10_alpha = S_10_new-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_00_new)));
      S_00_alpha = S_00_new-(S_00_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'*S_00_new)));
      newbeta = (S_11_alpha)\(S_10_alpha*alpharbar);
      %
      % Computing restricted newbeta estimate using R0tilde and R1tilde
      %
      R0tilde = alpharbar'*(eye(n)-(S_00_new*alpharort*((alpharort'*S_00_new*alpharort)\(alpharort'))))*newR0;
      R1tilde = newR1-(S_10_new*alpharort*((alpharort'*S_00_new*alpharort)\alpharort'))*newR0;
      [thisbeta,sig_beta,infmat,newabort] = linrest(R0tilde,R1tilde,newbeta',bRmat,brvec,0,1);
      newbeta = thisbeta';
      [alphar,Sigma_alpha,infomat,newabort,inv_Omega] = linrest(newR0,newbeta'*newR1,alphar,Rmat,rvec,0,1);
      %
      % create new R0 and R1 series using Psir, alphar, and newbeta
      %
      S_00_new = (1/T)*newR0*newR0';
      S_10_new = (1/T)*newR1*newR0';
      S_11_new = (1/T)*newR1*newR1';
      S_01_new = (1/T)*newR0*newR1';
      neweps = newR0-(alphar*newbeta'*newR1);
      newlnl = loglike(neweps);
      newOmega = (1/T)*neweps*neweps';
      if curr_ini.betagrad==0;
         grad = abs((newlnl-oldlnl)/oldlnl);
         oldgrad = 0;
      else;
         S_00_new = (1/T)*newR0*newR0';
         S_10_new = (1/T)*newR1*newR0';
         S_11_new = (1/T)*newR1*newR1';
         S_01_new = (1/T)*newR0*newR1';
         oldgrad = T*(vec(alphar))'*kron(eye(r),inv_Omega*(S_01_new-(alphar*newbeta'*S_11_new)))*bH;
         oldgrad = max(abs(oldgrad));
         if oldgrad>min([ccrit*10 1.0e-006]);
            oldgrad = 0;
         end;
         if det(newOmega)<1.0e-014;
            inv_newOmega = 100000*newOmega;
            inv_newOmega = inv_newOmega\eye(n);
            inv_newOmega = 100000*inv_newOmega;
         else;
            inv_newOmega = newOmega\eye(n);
         end;
         grad = T*(vec(alphar))'*kron(eye(r),inv_newOmega*(S_01_new-(alphar*newbeta'*S_11_new)))*bH;
      end;
      grad = abs(max(abs(grad))-oldgrad);
      infomat = [it grad];
      it = it+1;
   end;
   %
   % recompute ths conditional standard errors for beta
   %
   for i=1:r;
      %
      % Selecting the restrictions from H
      %
      si = sivec(i,1);
      if si>=2;
         sj = size(H,2);
         if i==1;
            Hsel = [eye(si);zeros(sj-si,si)];
         elseif i==r;
            Hsel = [zeros(sj-si,si);eye(si)];
         else;
            sk = sivec(1,1);
            for j=2:i-1;
               sk = sk+sivec(j,1);
            end;
            Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
         end;
         Hi = H*Hsel;
         %
         % Determining the normalizations
         %
         normelem = normvec(i,1);
         myexit = 0;
         for j=1:si;
            %
            % Fix Hi to make sure that the normalizing row
            % contains one unit element and zeros
            %
            normmatrix = Hi(normelem,:);
            for k=1:(n+q1+dcoint);
               if k~=normelem;
                  checkHi = [normmatrix;Hi(k,:)];
                  if rank(checkHi*checkHi')==size(checkHi,1);
                     normmatrix = checkHi;
                     if size(normmatrix,1)==size(Hi,2);
                        break;
                     end;
                  end;
               end;
            end;
            Hi = Hi/normmatrix;
            check = Hi(normelem,j);
            if (abs(check)>0)&(check~=1);
               Hi(:,j) = (1/Hi(normelem,j))*Hi(:,j);
               check = Hi(normelem,j);
            end;          
            if check==1;
               %
               % Split Hi into hi and H_i
               %
               if j==1;
                  hi = Hi(:,1);
                  H_i = Hi(:,2:si);
               elseif j==si;
                  hi = Hi(:,si);
                  H_i = Hi(:,1:si-1);
               else;
                  hi = Hi(:,j);
                  H_i = [Hi(:,1:j-1) Hi(:,j+1:si)];
               end;
               %
               % Check H_i to make sure it is of full rank
               %
               check2 = H_i'*H_i;
               if abs(det(check2))>=.000000001;
                  myexit=1;
               end;
            end;
            if myexit==1;
               break;
            end;
         end;
         %
         % Computing the asymptotic covariance for the
         % si-1 free parameters of column i in betar
         %
         information = alphar(:,i)'*inv_newOmega*alphar(:,i)*H_i'*S_11_alpha*H_i;
         covbeta = (1/T)*H_i*inverse(information)*H_i';
         %
         % Computing the asyptotic standard errors and
         % including them in stdbeta
         %
         newstdbeta(:,i) = sqrt(abs(diag(covbeta)));
      end;
   end;
end;

%
% end of rec_alpha_beta_restrict.m
%
