%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for computing recursive estimates     %
%          of the non-zero eigenvalues for I(1) cointegration    %
%          models - based on Hansen & Johansen (1999), "Some     %
%          tests for parameter constancy in cointegrated         %
%          VAR-models", Econometrics Journal, 2, 306-333         %
%                                                                %
%          Function: rectlambda                                  %
%                                                                %
%          Input:                                                %
%                        Z0: nxT matrix                          %
%                        Z1: nq1xT matrix                        %
%                        Z2: n(p-1)+d matrix                     %
%                        r: cointegration rank                   %
%                        p: lag order                            %
%                        d: number of deterministic variables    %
%                        T0: integer between (1/3) and           %
%                           (defaults to (2/3)                   %
%                        np: number of parameters per equation   %
%                        deterselect: 1x4 0-1 vector describing  %
%                           deterministic model                  %
%                        waitbar: indicator if a progress        %
%                           dialog should be displayed or not    %
%                           (default=0, no progress dialog       %
%                                                                %
%          Output:                                               %
%                        lambdaT: nx(T-T1+1) matrix with         %
%                            eigenvalues                         %
%                        lambdastd: nx(T-T1+1) matrix with       %
%                            asymptotic variances for the        %
%                            eigenvalues                         %
%                        sigmastd: nx(T-T1+1) matrix with        %
%                            asymptotic variances for the        %
%                            transformed eigenvalues             %
%                            ln(eigi/(1-eigi))                   %
%                        tauT = nx(T-T1+1) matrix with           %
%                            fluctuation tests                   %
%                        nyblomT = 1x(T-T1+1) matrix with        %
%                            recursive Nyblom tests when         %
%                            r<n (else nyblom = 0)               %
%                        nyblomcorr = like nyblom but corrected  %
%                            for degrees of freedom              %
%                        xisumT = 1x(T-T1+1) matrix with         %
%                            recursive estimates of the sum of   %
%                            xi(i)=log(lambda(i)/[1-lambda(i)])  %
%                        xistdT = 1x(T-T1+1) matrix with         %
%                            estimated asymptotic variances for  %
%                            xisumT                              %
%                        evaluet: 1x(T-T1+1) matrix with the     %
%                            estimated largest non-unit          %
%                            eigenvalue of the companion matrix  %
%                        tT = 1x(T-T1+1) matrix with t/T         %
%                        abort: text string indicating if        %
%                            the iterations have been cancelled  %
%                            or not                              %
%                                                                %
%          NOTES: Requires q0=q1=0. Moreover, the short run      %
%                 dynamics are NOT fixed at the full sample      %
%                 horizon. Finally, all eigenvalues of the       %
%                 estimated companion matrix must be             %
%                 non-explosive                                  %
%                                                                %
%          This version:        March 5, 2008                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2008          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [lambdaT,lambdastd,sigmastd,tauT,nyblomT,nyblomcorr,xisumT,xistdT,evaluet,tT,abort] = rectlambda(Z0,Z1,Z2,r,p,d,T0,np,deterselect,waitbar)

curr_ini = find_inisettings;

[n T] = size(Z0);

%
% Ensuring that all input variables are set
%

if nargin<7;
   T1 = round(2*T/3);
else;
   if T0<0;
      T1 = round(2*T/3);
   elseif T0>0.95;
      T1 = round(2*T/3);
   else;
      T1 = round(T*T0);
   end;
end;

if T1<=size(Z1,1)+size(Z2,1);
   T1 = size(Z1,1)+size(Z2,1)+1;
end;
%
% check if Z1 has a zero row
%
ZeroRows = [];
for i=1:size(Z1,1);
   if isequal(Z1(i,1:T1),zeros(1,T1))==1;
      ZeroRows = [ZeroRows i];
   end;
end;
%
% if a zero row is located, we replace it with 1-rowvalue
%
if isempty(ZeroRows)==0;
   Z1(ZeroRows,:) = ones(length(ZeroRows),T)-Z1(ZeroRows,:);
   if length(ZeroRows)>1;
      %
      % we need to try to move T1 up
      %
      for t=T1:T;
         if rank(Z1(:,1:t))==size(Z1,1);
            T1 = t;
            break;
         end;
      end;
   end;
end;
%
if nargin<8;
   np = r+round(r*(n-r)/n)+n*(p-1)+d;
end;

if nargin<9;
   deterselect = [1 0 0 0];
end;

if nargin<10;
   waitbar = 0;
end;

dcoint = 0;
if ((deterselect(1,2)==1)|(deterselect(1,4)==1));
   dcoint = 1;
end;

if T1>T;
   lambdaT = [];
   lambdastd = [];
   sigmastd = [];
   tauT = [];
   nyblomT = [];
   nyblomcorr = [];
   xisumT = [];
   xistdT = [];
   evaluet = [];
   tT = [];
   abort = '';
   return;
end;

if waitbar==1;
   realscrsz = get(0,'ScreenSize');
   input_var.title = ['Progress for ' int2str(T-T1+1) ' recursive estimates'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.label = 'Recursion number:';
   input_var.stop = 0;
   input_var.clock = 0;
   input_var.name = 'Recursive Estimation of Lambda';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','Recursive Estimation of Lambda');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
   progressdlg([(1/(T-T1+1)) 1]);
end;

%
% compute full sample estimates
%

[S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,trace_test] = mlcointbasic(Z0,Z1,Z2,r);

lambdaT = lambda(1:r,1);
tauT = zeros(r,1);
lambdaT0 = lambda(1:r,1);

alpha = S_01*beta*inverse(beta'*S_11*beta);
if isempty(Z2)==0;
   Psi = M_02*inverse(M_22)-(alpha*beta'*M_12*inverse(M_22));
   epsilon = Z0-(alpha*beta'*Z1)-(Psi*Z2);
else;
   Psi = [];
   epsilon = Z0-(alpha*beta'*Z1);
end;
Omega = (1/T)*epsilon*epsilon';

%
% material for Nyblom test
%

if r<n;
   beta_orth = null(beta(1:n,1:r)');
   if (deterselect(1,1)|deterselect(1,3))==1;
      c = beta;
      c_orth = beta_orth;
   else;
      c = beta(1:n,:);
      c_orth = beta_orth(1:n,:);
      c = [c;zeros(1,r)];
      c_orth = [[c_orth zeros(n,1)];[zeros(1,n-r) 1]];
   end;
   s1 = size(c_orth,1);
   if curr_ini.taylor_nyblom==1;
      c_bar = c*inverse(c'*c);
      c_bar_orth = c_orth*inverse(c_orth'*c_orth);
      betaT = beta;
      alpha_cT = alpha*betaT'*c_bar;
      M_T = (1/T)*c_orth'*S_11(1:s1,1:s1)*c_orth;
      V_T = alpha_cT'*inverse(Omega)*alpha_cT;
   else;
      J_T = kron(alpha'*inverse(Omega)*alpha,c_orth'*S_11(1:s1,1:s1)*c_orth);
      alpha_T = alpha;
      Omega_T = Omega;
      beta_T = beta(1:s1,1:r);
   end;
end;

beta = beta(1:n,1:r);

%
% computing the max non-unit full-sample eigenvalue of the companion matrix
%

eigent = companionrest(alpha,Psi,beta,p,n,d);
eigent = abs(sort(eigent));
i = size(eigent,1);
while i>=1;
   evaluet = eigent(i,1);
   if strcmp(num2str(evaluet,'%1.10f'),'1.0000000000')==1;
      i = i-1;
   else;
      i = 0;
   end;
end;

%
% computing the matrix A
%

if p>=2;
   Phi = Psi(:,d+1:d+n*(p-1));
end;

if p>=2;
   A = [[(beta'*alpha)+eye(r) beta'*Phi];[alpha Phi]];
   if p>=3;
      A = [A;[zeros((p-2)*n,r) eye((p-2)*n) zeros((p-2)*n,n)]];
   end;
else;
   A = [[(beta'*alpha)+eye(r) zeros(r,n)];[alpha zeros(n,n)]];
end;

OmegaTilde = [[beta'*Omega*beta beta'*Omega];[Omega*beta Omega]];

if p>=3;
   OmegaTilde = [[OmegaTilde zeros(r+n,n*(p-2))];zeros(n*(p-2),r+n*(p-1))];
end;

%
% Computing the covariance matrix SigmaTilde
%

q = size(A,1);
Ak = eye(q*q)-kron(A,A);
sigmatilde = Ak\vec(OmegaTilde);
SigmaTilde = vecinv(sigmatilde,q);
SigmaTildeInv = eye(q)/SigmaTilde;

%
% Computing asymptotic variances
%

Upsilon = kron(SigmaTildeInv,SigmaTildeInv)/Ak;
Upsilon2 = Upsilon+Upsilon';

lambdastd = zeros(r,1);
sigmastd = zeros(r,1);
xistdT = 0;
tT = 1;

for i=1:r;
   itilde = i+(i-1)*(r+n*(p-1));
   lambdai = (1-lambda(i,1))*lambda(i,1);
   lambdastd(i,1) = (2/T)*lambdai*lambdai*Upsilon2(itilde,itilde);
   sigmastd(i,1) = (2/T)*Upsilon2(itilde,itilde);
   for j=1:r;
      jtilde = j+(j-1)*(r+n*(p-1));
      xistdT = xistdT+(Upsilon2(itilde,jtilde));
   end;
end;

lambdastdT = lambdastd;
sigmastdT = sigmastd;

xistdT = (2/T)*xistdT;

nyblomT = 0;
nyblomcorr = 0;

abort = '';
t = T-1;
while t>=T1;
   if waitbar==1;
      abort = get(mywait,'UserData');
      if strcmp(abort,'cancel')==1;
         break;
      else;
         progressdlg([((T-t+1)/(T-T1+1)) (T-t+1)]);
      end;
   end;
   %
   % Checking for affinely dependent deterministic variables
   %
%   if d>0;
      [Z2t,dfix] = fixdet(Z1(:,1:t),Z2(:,1:t),d,dcoint);
%   else;
%      dfix = 0;
%      if isempty(Z2)==0;
%         Z2t = Z2(:,1:t);
%      else;
%         Z2t = [];
%      end;
%   end;
   %
   % Reestimating the unrestricted beta for period t
   %
   [S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,trace_test] = mlcointbasic(Z0(:,1:t),Z1(:,1:t),Z2t,r);
   if isempty(Z2)==0;
      M_01 = (1/t)*Z0(:,1:t)*Z1(:,1:t)';
      M_11 = (1/t)*Z1(:,1:t)*Z1(:,1:t)';
      M_21 = (1/t)*Z2t*Z1(:,1:t)';
   end;
   %
   % Doing the rest
   %
   if curr_ini.transeig==1;
      tauTt = log((lambda(1:r,1)./(ones(size(lambdaT0,1),1)-lambda(1:r,1))))-log((lambdaT0./(ones(size(lambdaT0,1),1)-lambdaT0)));
   else;
      tauTt = lambda(1:r,1)-lambdaT0;
   end;
   if abs(max(lambda(1:r,1))-1)<1.0e-5;
      for i=1:r;
         if abs(lambda(i,1)-1)<1.0e-5;
            lambda(i,1) = 0.99999;
         end;
      end;
   end;
   %
   lambdaT = [lambda(1:r,1) lambdaT];
   alpha = S_01*beta*inverse(beta'*S_11*beta);
   Omega = S_00-(alpha*beta'*S_11*beta*alpha');
   if isempty(Z2)==0;
      Psi = M_02*inverse(M_22)-(alpha*beta'*M_12*inverse(M_22));
   else;
      Psi = [];
   end;
   %
   if r<n;
      if curr_ini.taylor_nyblom==1;
         betat = beta;
         M_t = c_orth'*S_11(1:s1,1:s1)*c_orth;
         beta_ct = betat*inverse(c_bar'*betat);
         q_t = c_bar_orth'*(beta_ct-betaT);
         nyblomt = V_T*q_t'*M_t*inverse(M_T)*M_t*q_t;
         nyblomt = sum(diag(nyblomt));
         nyblomT = [(t/T)*(t/T)*nyblomt(1,1) nyblomT];
         nyblomcorr = [((T-np)/T)*(t/T)*(t/T)*nyblomt(1,1) nyblomcorr];
      else;
         S_t = (vec(alpha_T))'*kron(eye(r),inverse(Omega_T)*(S_01(:,1:s1)-alpha_T*beta_T'*S_11(1:s1,1:s1))*c_orth);
         Q_t = (t/T)*(t/T)*T*S_t*inverse(J_T)*S_t';
         nyblomT = [Q_t nyblomT];
         nyblomcorr = [((T-np)/T)*Q_t nyblomcorr];
      end;
   end;
   %
   beta = beta(1:n,1:r);
   %
   % Computing the max non-unit eigenvalue of the companion matrix
   %
   eigen_t = companionrest(alpha,Psi,beta,p,n,dfix);
   eigen_t = abs(sort(eigen_t));
   i = size(eigen_t,1);
   while i>=1;
      evalue_t = eigen_t(i,1);
      if strcmp(num2str(evalue_t,'%1.10f'),'1.0000000000')==1;
         i = i-1;
      else;
         i = 0;
      end;
   end;
   evaluet = [evalue_t evaluet];
   %
   % computing the matrix A
   %
   if p>=2;
      Phi = Psi(:,dfix+1:dfix+n*(p-1));
   end;
   if p>=2;
      A = [[(beta'*alpha)+eye(r) beta'*Phi];[alpha Phi]];
      if p>=3;
         A = [A;[zeros((p-2)*n,r) eye((p-2)*n) zeros((p-2)*n,n)]];
      end;
   else;
      A = [[(beta'*alpha)+eye(r) zeros(r,n)];[alpha zeros(n,n)]];
   end;
   OmegaTilde = [[beta'*Omega*beta beta'*Omega];[Omega*beta Omega]];
   if p>=3;
      OmegaTilde = [[OmegaTilde zeros(r+n,n*(p-2))];zeros(n*(p-2),r+n*(p-1))];
   end;
   %
   % Computing the covariance matrix SigmaTilde
   %
   Ak = eye(q*q)-kron(A,A);
%   if (max(max(isinf(Ak)))==0)&(max(max(isnan(Ak)))==0)&(rank(Ak,1.0e-7)==q*q);
      sigmatilde = Ak\vec(OmegaTilde);
      SigmaTilde = vecinv(sigmatilde,q);
      SigmaTildeInv = eye(q)/SigmaTilde;
      %
      % Computing asymptotic variances
      %
      Upsilon = kron(SigmaTildeInv,SigmaTildeInv)/Ak;
      Upsilon2 = Upsilon+Upsilon';
      lambdastdt = zeros(r,1);
      sigmastdt = zeros(r,1);
      taut = zeros(r,1);
      xistdt = 0;
      for i=1:r;
         itilde = i+(i-1)*(r+n*(p-1));
         lambdai = (1-lambda(i,1))*lambda(i,1);
         lambdastdt(i,1) = (2/T)*lambdai*lambdai*Upsilon2(itilde,itilde);
         sigmastdt(i,1) = (2/T)*Upsilon2(itilde,itilde);
         if curr_ini.transeig==1;
            taut(i,1) = abs((t/T)*(tauTt(i,1)/sqrt(sigmastdT(i,1))));
         else;
            taut(i,1) = abs((t/T)*(tauTt(i,1)/sqrt(lambdastdT(i,1))));
         end;
         for j=1:r;
            jtilde = j+(j-1)*(r+n*(p-1));
            xistdt = xistdt+(Upsilon2(itilde,jtilde));
         end;
      end;
%   else;
%      xistdt = xistdT(:,1);
%      taut = tauT(:,1);
%      lambdastdt = lambdastd(:,1);
%      sigmastdt = sigmastd(:,1);
%   end;
   xistdT = [(2/T)*xistdt xistdT];
   tT = [(t/T) tT];
   tauT = [taut tauT];
   lambdastd = [lambdastdt lambdastd];
   sigmastd = [sigmastdt sigmastd];
   t = t-1;
end;

if r>1;
   xisumT = sum(log(lambdaT./(1-lambdaT)));
else;
   xisumT = log(lambdaT./(1-lambdaT));
end;

if waitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

%
% End of function rectlambda.m
%
