%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab control file for estimating a structural VAR   %
%          model with cointegration restrictions and identifying %
%          restrictions on the contemporaneous and the long      %
%          run responses                                         %
%                                                                %
%          Function: varrun                                      %
%                                                                %
%          Input:                                                %
%                        cm: structure containing                %
%                           information about current model      %
%                        curr_ini: current options structure     %
%                        maingui: handle to Structural VAR GUI   %
%                                                                %
%          Output:                                               %
%                        none                                    %
%                                                                %
%          This version:        June 28, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function varrun(cm,curr_ini,maingui)

%
% make sure the Output tab is displayed when option is selected
%
if curr_ini.logtab==1;
   cf = findobj('Type','figure','Tag','StructuralVAR');
   if (isempty(cf)==0)&(ishandle(cf)==1);
      mod = get(cf,'UserData');
      h_log = mod.tab3;
      set(cf,'CurrentObject',h_log);
      maindlg('tab');
   end;
end;
%
% Display wait dialogue
%
txt = strvcat(' ','Initializing...','Reading data.','Please hold!');
mywait = waitdlg(txt,'information');
waitf = get(mywait,'UserData');

%
% get sample information
%

cm = getsample(cm,cm.endofile,cm.endosheet);

if isempty(cm.fp)==1;
   set(waitf.text,'UserData','done');
   delete(mywait);
   drawnow;
   txt = ['ERROR: Unable to read data from ' fix_path(getfilename(cm.endofile)) '.'];
   about(txt,'error','Bad data...',120,350);
   return;
end;

if curr_ini.critval==1;
   cm.critvalnum = 1;
elseif curr_ini.critval==2;
   cm.critvalnum = 2;
elseif curr_ini.critval==5;
   cm.critvalnum = 3;
elseif curr_ini.critval==10;
   cm.critvalnum = 4;
elseif curr_ini.critval==20;
   cm.critvalnum = 5;
else;
   cm.critvalnum = 6;
end;

%
%    Loading data
%

cm.x0 = loaddata(cm.endofile,cm.endosheet);

%
% Setting up deterministic component
%

if cm.deterselect(1,1)==1;
   cm.x1 = ones(size(cm.x0,1),1);
   cm.x1_r = [];
   cm.constvec = 'const';
   cm.restvec = '';
elseif cm.deterselect(1,2)==1;
   cm.x1 = [];
   cm.x1_r = ones(size(cm.x0,1),1);
   cm.constvec = '';
   cm.restvec = 'const';
elseif cm.deterselect(1,3)==1;
   cm.x1 = [ones(size(cm.x0,1),1) (1:size(cm.x0,1))'-max(cm.p,cm.pe)];
   cm.x1_r = [];
   cm.constvec = 'const trend';
   cm.restvec = '';
elseif cm.deterselect(1,4)==1;
   cm.x1 = ones(size(cm.x0,1),1);
   cm.x1_r = (1:size(cm.x0,1))'-max(cm.p,cm.pe);
   cm.constvec = 'const';
   cm.restvec = 'trend';
end;

cm.dcoint = 0;
if strcmp(cm.restvec,'')==0;
   cm.dcoint = 1;
end;

cm.ds = 0;
if cm.seasonal(1,1)==1;
   if strcmp(cm.freq,'')==0;
      [seamat,seastring] = seasonal_fcn(cm.freq,size(cm.x0,1));
      cm.x1 = [cm.x1 seamat];
      if strcmp(cm.constvec,'')==1;
         cm.constvec = seastring;
      else;
         cm.constvec = [cm.constvec ' ' seastring];
      end;
   end;
   cm.ds = size(seamat,2);
end;

%
% determining the number of automatically created deterministic variables
%
cm.ad = size(cm.x1,2);
cm.datad = 0;
if strcmp(cm.deterfile,'')==0;
   if fileexist(cm.deterfile)==1;
      dummyfile = loaddata(cm.deterfile,cm.detersheet);
      if isempty(dummyfile)==1;
         set(waitf.text,'UserData','done');
         delete(mywait);
         drawnow;
         txt = ['ERROR: Unable to read data from ' fix_path(getfilename(cm.deterfile)) '.'];
         about(txt,'error','Bad data...',120,350);
         return;
      end;
      if size(dummyfile,1)==size(cm.x1,1);
         cm.x1 = [cm.x1 dummyfile];
      elseif isempty(cm.x1)==1;
         cm.x1 = dummyfile;
      end;
      cm.datad = size(cm.x1,2)-cm.ad;
   end;
end;

if strcmp(cm.exo1file,'')==0;
   if fileexist(cm.exo1file)==1;
      cm.x2 = loaddata(cm.exo1file,cm.exo1sheet);
      if isempty(cm.x2)==1;
         set(waitf.text,'UserData','done');
         delete(mywait);
         drawnow;
         txt = ['ERROR: Unable to read data from ' fix_path(getfilename(cm.exo1file)) '.'];
         about(txt,'error','Bad data...',120,350);
         return;
      end;
   else;
      cm.x2 = [];
   end;
else;
   cm.x2 = [];
end;
if strcmp(cm.exo0file,'')==0;
   if fileexist(cm.exo0file)==1;
      cm.x3 = loaddata(cm.exo0file,cm.exo0sheet);
      if isempty(cm.x3)==1;
         set(waitf.text,'UserData','done');
         delete(mywait);
         drawnow;
         txt = ['ERROR: Unable to read data from ' fix_path(getfilename(cm.exo0file)) '.'];
         about(txt,'error','Bad data...',120,350);
         return;
      end;
   else;
      cm.x3 = [];
   end;
else;
   cm.x3 = [];
end;

if size(cm.x0,2)>size(cm.x0,1);
   cm.x0 = cm.x0';
end;

if size(cm.x1,2)>size(cm.x1,1);
   cm.x1 = cm.x1';
end;

if size(cm.x2,2)>size(cm.x2,1);
   cm.x2 = cm.x2';
end;

if size(cm.x3,2)>size(cm.x3,1);
   cm.x3 = cm.x3';
end;

if isempty(cm.x2)==0;
   origq1 = size(cm.x2,2);
else;
   origq1 = 0;
end;

extrai0 = 0;
if (isempty(cm.x3)==0)&(curr_ini.rahbek==1);
   if (size(cm.x2,1)==size(cm.x3,1))|(isempty(cm.x2)==1);
      extrai0 = size(cm.x3,2);
      cm.x3 = cumsum(cm.x3);
      cm.x2 = [cm.x2 cm.x3];
      cm.x3 = [];
   end;
end;

%
% Setting up the exogenous variables into the cm.x2 matrix
% and determining the number of I(1) variables
%

if isempty(cm.x3)==0;
   if size(cm.x2,1)==size(cm.x3,1);
      cm.q1 = size(cm.x2,2);
      cm.x2 = [cm.x2 cm.x3];
   else;
      cm.q1 = 0;
      cm.x2 = cm.x3;
   end;
else;
   if isempty(cm.x2)==0;
      cm.q1 = size(cm.x2,2);
   else;
      cm.q1 = 0;
   end;
end;

%
% setting up subsample
%

if size(cm.x1,2)>0;
   if cm.lp<=size(cm.x1,1);
      cm.x1 = cm.x1(cm.fp:cm.lp,:);
   else;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
      txt = ['ERROR: Unable to match observations in ' fix_path(getfilename(cm.deterfile)) ' with those in ' fix_path(getfilename(cm.endofile)) '. Please make sure that the observations match in all your data-files'];
      about(txt,'error','Data mismatch...',120,400);
      return;
   end;
end;
if size(cm.x2,1)>1;
   if cm.lp<=size(cm.x2,1);
      cm.x2 = cm.scale_endo*cm.x2(cm.fp:cm.lp,:);
   else;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
      if (strcmp(strtrim(cm.exo1file),'')==0)&(strcmp(strtrim(cm.exo0file),'')==1);
         txt = ['ERROR: Unable to match observations in ' fix_path(getfilename(cm.exo1file)) ' with those in ' fix_path(getfilename(cm.endofile)) '. Please make sure that the observations match in all your data-files'];
      elseif (strcmp(strtrim(cm.exo1file),'')==1)&(strcmp(strtrim(cm.exo0file),'')==0);
         txt = ['ERROR: Unable to match observations in ' fix_path(getfilename(cm.exo0file)) ' with those in ' fix_path(getfilename(cm.endofile)) '. Please make sure that the observations match in all your data-files'];
      else;
         txt = ['ERROR: Unable to match observations in ' fix_path(getfilename(cm.exo0file)) ' and in ' fix_path(getfilename(cm.exo1file)) ' with those in ' fix_path(getfilename(cm.endofile)) '. Please make sure that the observations match in all your data-files'];
      end;
      about(txt,'error','Data mismatch...',120,400);
      return;
   end;
end;
if isempty(cm.x1_r)==0;
   cm.x1_r = cm.x1_r(cm.fp:cm.lp,:);
end;
cm.x0 = cm.scale_endo*cm.x0(cm.fp:cm.lp,:);

%
% Checking names of variables and creating ones for the missing
% entries
%

if strcmp(cm.evars,'')==1;
   cm.n = size(cm.x0,2);
   i = 1;
   while i<=cm.n;
      if i==1;
         cm.evars = 'y_1';
      else;
         cm.evars = [cm.evars ' y_' num2str(i)];
      end;
      i = i+1;
   end;
end;

cm.dummies = 0;
cm.d = size(cm.x1,2);
if strcmp(cm.deterfile,'')==0;
   cm.dummies = 1;
   if strcmp(cm.deter,'')==1;
      if cm.deterselect(1,1)==1;
         fixed = 1;
      elseif cm.deterselect(1,2)==1;
         fixed = 0;
      elseif cm.deterselect(1,3)==1;
         fixed = 2;
      elseif cm.deterselect(1,4)==1;
         fixed = 1;
      end;
      for i=1:cm.d-fixed;
         if i==1;
            cm.deter = 'd_1';
         else;
            cm.deter = [cm.deter ' d_' num2str(i)];
         end;
      end;
   end;
end;

if strcmp(cm.constvec,'')==0;
   cm.deter = [cm.constvec ' ' cm.deter];
end;

[cm.x1,cm.deter,cm.deterdel] = checkdeter(cm.x1,cm.x1_r,cm.deter);
cm.d = size(cm.x1,2);
if cm.deterselect(1,3)==1;
   cm.dd = cm.d-1;
else;
   cm.dd = cm.d;
end;

if strcmp(cm.xvars1,'')==1;
   i = 1;
   while i<=origq1;
      if i==1;
         cm.xvars1 = 'x1_1';
      else;
         cm.xvars1 = [cm.xvars1 ' x1_' num2str(i)];
      end;
      i = i+1;
   end;
end;

if strcmp(cm.xvars0,'')==1;
   i = 1;
   cm.q = size(cm.x2,2);
   while i<=cm.q-cm.q1;
      if i==1;
         cm.xvars0 = 'x0_1';
      else;
         cm.xvars0 = [cm.xvars0 ' x0_' num2str(i)];
      end;
      i = i+1;
   end;
end;

%
% extra check in case cm.x3 moved to cm.x2 (Rahbek&Mosconi fix)
%

if strcmp(cm.xvars0,'')==1;
   if extrai0>0;
      for i=1:extrai0;
         if i==1;
            cm.xvars0 = 'x0_1';
         else;
            cm.xvars0 = [cm.xvars0 ' x0_' num2str(i)];
         end;
      end;
   end;
end;

if (curr_ini.rahbek==1)&(isempty(cm.x2)==0);
   if strcmp(cm.xvars0,'')==0;
      cm.xvars0 = stradd(cm.xvars0,'cum');
      if strcmp(cm.xvars1,'')==1;
         cm.xvars1 = cm.xvars0;
      else;
         cm.xvars1 = [cm.xvars1 ' ' cm.xvars0];
      end;
      cm.xvars0 = '';
   end;
end;

%
% Creating data matrices in VECM form
%
if isempty(cm.x2)==1;
   [cm.Z0,cm.Z1,cm.Z2,cm.Z1t]=vecmdata(cm.x0,cm.p,cm.x1);
%   if cm.cointvalue==0;
      [cm.X0,cm.X1] = vardata(cm.x0,cm.p,[cm.x1 cm.x1_r]);
%   end;
   cm.q = 0;
   cm.q1 = 0;
   cm.q0 = 0;
   exo = 0;
else;
   [cm.Z0,cm.Z1,cm.Z2,cm.Z1t]=vecmdata(cm.x0,cm.p,cm.x1,cm.x2,cm.q1,cm.pe);
%   if cm.cointvalue==0;
      [cm.X0,cm.X1] = vardata(cm.x0,cm.p,[cm.x1 cm.x1_r],cm.x2,cm.q1,cm.pe);
%   end;
   cm.q = size(cm.x2,2);
   cm.q0 = cm.q-cm.q1;
   exo = 1;
end;

cm.old_x1_r = cm.x1_r;
if isempty(cm.x1_r)==0;
   num = size(cm.x1_r,1);
   cm.x1_r = cm.x1_r(max(cm.p,cm.pe)+1:num,:);
   cm.Z1 = [cm.Z1;cm.x1_r'];
   if max(cm.x1_r)==min(cm.x1_r);
      cm.Z1t = [cm.Z1t;cm.x1_r'];
   else;
      cm.Z1t = [cm.Z1t;cm.x1_r'+1];
   end;
end;

[cm.n cm.T] = size(cm.Z0);
nq1 = size(cm.Z1,1);
nz2 = size(cm.Z2,1);
if cm.T<=nq1+nz2;
   if isempty(mywait)==0;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
   end;
   txt = ['The model you have specified has ' num2str(nq1+nz2,'%0.0f') ' parameters per equation and only ' num2str(cm.T,'%0.0f') ' observations. You must respecify the model before it can be estimated.'];
   about(txt,'warning','Warning - Too many parameters',120,400);
   return;
end;

%
% check if we can invert the product moment matrices
%
M00 = (1/cm.T)*cm.Z0*cm.Z0';
M11 = (1/cm.T)*cm.Z1*cm.Z1';
if isempty(cm.Z2)==0;
   M22 = (1/cm.T)*cm.Z2*cm.Z2';
   M33 = (1/cm.T)*[cm.Z2;cm.Z1]*[cm.Z2;cm.Z1]';
else;
   M22 = 1;
   M33 = [];
end;
if (max(max(isnan(M00)))==1)|(max(max(isnan(M11)))==1)|(max(max(isnan(M22)))==1);
   if isempty(mywait)==0;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
   end;
   txt = 'There has been some errors reading your data. NaN''s were detected in the product moments matrices. Please try running the estimation routine again or check your data.';
   about(txt,'warning','Warning - NaNs Detected',120,400);
   return;
end;
%
checks = zeros(1,3);
if max(size(M00))>rank(M00);
   checks(1,1) = 1;
end;
if max(size(M11))>rank(M11);
   checks(1,2) = 1;
end;
if max(size(M22))>rank(M22);
   checks(1,3) = 1;
end;
check3 = 0;
if isempty(M33)==0;
   if (max(size(M33))>rank(M33))&(max(size(M33))>rank(M33,eps));
      check3 = 1;
   elseif (max(size(M33))>rank(M33))&(max(size(M33))==rank(M33,eps));
      check3 = 2;
   end;
end;
if check3==1;
   if isempty(mywait)==0;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
   end;
   txt ='The model you have specified cannot be estimated. Structural VAR has found multicollinearity among the regressors.';
   about(txt,'warning','Warning - Linear Dependence',100,400);
   return;
elseif check3==2;
   if isempty(mywait)==0;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
   end;
   txt ='Structural VAR has found bad scaling among the variabels. The model you have specified will not be estimated on your data since numerical precision will be extremely poor.';
   about(txt,'warning','Warning - Bad Scaling',120,400);
   return;
else;
   if sum(checks)>0;
      if isempty(mywait)==0;
         set(waitf.text,'UserData','done');
         delete(mywait);
         drawnow;
      end;
      txt = strvcat('The model you have specified cannot be estimated. The following product moment matrices are non-invertible:',' ');
      if checks(1,1)==1;
         txt = strvcat(txt,'          * First differences of endogenous variables');
      end;
      if checks(1,2)==1;
         txt = strvcat(txt,'          * Lagged endogenous variables in levels');
      end;
      if checks(1,3)==1;
         txt = strvcat(txt,'          * Lagged first differences and deterministics');
      end;
      txt = strvcat(txt,' ','Your data thus contains linearly dependent variables. Please make sure to remove the redundant variables.');
      about(txt,'warning','Warning - Linear Dependence',120+((sum(checks)-1)*10)+(max((sum(checks)-2)*5,0)),600);
      return;
   end;
end;

cm.outputdir = getpath(cm.outputfile);
if ispc==1;
   if strcmp(cm.outputdir(1,size(cm.outputdir,2)),'\')==0;
      cm.outputdir = [cm.outputdir '\'];
   end;
else;
   if strcmp(cm.outputdir(1,size(cm.outputdir,2)),'/')==0;
      cm.outputdir = [cm.outputdir '/'];
   end;
end;
cm.myvars = vecstr(cm.evars);
cm.mv2add = ' ';
if size(cm.myvars,2)<10;
   for i=2:10-size(cm.myvars,2);
      cm.mv2add = [cm.mv2add ' '];
   end;
end;

if cm.q1==0;
   cm.xvars1 = '';
end;

if (cm.q-cm.q1)==0;
   cm.xvars0 = '';
end;

cm.r = cm.n;

%
% check if we can use asymptotic distributions with nyblom tests
%

cm.nyblom_asymp = 0;
if (cm.dummies==0)&(cm.q1==0)&(cm.q0==0)&(cm.n<=12);
   cm.nyblom_asymp = 1;
end;

%
% check if the model has dummy variables
%

cm.isdummy = [];
if cm.datad>0;
   cm.isdummy = isdummy(cm.Z2(cm.ad+1:cm.d,:));
end;

%
% if curr_ini.method is 3-step, 2-step or 1-step Classical Gaussian ML
%

if (curr_ini.method(1,1)==1)|(curr_ini.method(1,2)==1);
   %
   % Run basic ML cointegration analysis to get rank tests
   % and eigenvalues and input matrices for estimation
   % These results are computed even if no cointegration
   % has been selected since no rank reduction constraint
   % is imposed yet
   %
   %
   % Adjust cm.samplebeg to fit with first observation
   %
   cm.oldsamplebeg = cm.samplebeg;
   if strcmp(cm.freq,'m')==1;
      cm.samplebeg = cm.samplebeg+(max(cm.p,cm.pe)/12);
   elseif strcmp(cm.freq,'q')==1;
      cm.samplebeg = cm.samplebeg+(max(cm.p,cm.pe)/4);
   else;
      cm.samplebeg = cm.samplebeg+max(cm.p,cm.pe);
   end;
   lag_str = 'lags';
   if cm.p==1;
      lag_str = 'lag';
   end;
   [initfirst,initlast,samplefirst,samplelast] = sampledates(cm.oldsamplebeg,cm.sampleend,max(cm.p,cm.pe),cm.freq);
   if cm.q1+cm.q0>0;
      add_lag_str = ['Using ' num2str(cm.pe) ' lag'];
      if cm.pe~=1;
         add_lag_str = [add_lag_str 's'];
      end;
      add_lag_str = [add_lag_str ' for exogenous variables...'];
   else;
      add_lag_str = '';
   end;
   setlogtext(strvcat('Initializing Structural VAR estimation...',' ',['Estimation sample: ' samplefirst ' - ' samplelast '...'],['Initial sample:    ' initfirst ' - ' initlast '...'],['Using ' num2str(cm.p,'%0.0f') ' ' lag_str ' for endogenous variables...'],add_lag_str,' '),maingui);
   if cm.cointvalue==1;
      txt = strvcat(' ','Initializing...','Computing trace tests.','Please hold!');
      set(waitf.text,'String',txt);
      setlogtext('Computing trace tests...',maingui);
   end;
   cm.bartcorr = [];
   [cm.S_00,cm.S_01,cm.S_10,cm.S_11,cm.M_02,cm.M_22,cm.M_12,cm.beta,cm.lambda,cm.trace] = mlcointbasic(cm.Z0,cm.Z1,cm.Z2,cm.n);
   %
   if ((cm.q1+cm.q0==0)&(curr_ini.tracebart==1)&(cm.dd-cm.ds<2)&(cm.cointvalue==1));
      cm.bartcorr = barttr(cm.S_00,cm.S_01,cm.S_10,cm.S_11,cm.M_02,cm.M_22,cm.M_12,cm.beta,cm.d,cm.p,cm.T,cm.deterselect,waitf);
      cm.trace = cm.trace./cm.bartcorr;
   end;
   %
   savebeta = cm.beta;
   cm.oldbeta = cm.beta;
   cm.stdbeta = [];
   %
   % initial estimates of the parameters
   %
   cm.alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
   if isempty(cm.Z2)==0;
      cm.Psi = (cm.M_02/cm.M_22)-(cm.alpha*cm.beta'*(cm.M_12/cm.M_22));
      cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1)-(cm.Psi*cm.Z2);
   else;
      cm.Psi = [];
      cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1);
   end;
   cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
   %
   eigenvalues = companion(cm.Z0,cm.Z1,cm.Z2,cm.n,cm.p,cm.n,cm.q1,cm.q0,cm.d,cm.T);   
   [eigenvalues2,AscInd] = sort(abs(eigenvalues));
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i = 1;
   while i<=nr;
      DescInd(i)=AscInd(nr+1-i);
      i =i+1;
   end;
   eigenvalues = eigenvalues(DescInd);
   cm.evalue = max(abs(eigenvalues));
   %
   cm.df = cm.n*cm.p+cm.q1*(cm.pe+1)+cm.q0*(cm.pe+1)+cm.d+cm.dcoint;
   cm.dfbeta = 0;
   cm.dfalpha = 0;
   cm.restrict = zeros(1,2);
   %
   % write results to output file
   %
   if curr_ini.noprint==0;
      sp_str = spacestr(curr_ini.decimals-2);
      if curr_ini.append==1;
         fileformat = 'at';
      else;
         fileformat = 'wt';
      end;
      fout = fopen(cm.outputfile,fileformat);
      if curr_ini.texoutputformat>=2;
         if strcmp(fileformat,'wt')==1;
            fprintf(fout,'\n');
            for i=1:size(curr_ini.texheadertxt,1);
               fprintf(fout,[strrep(strtrim(curr_ini.texheadertxt(i,:)),'\','\\') '\n']);
            end;
            fprintf(fout,'\n\\newcommand{\\header}[1]{\\vspace*{3mm}\\noindent\\textbf{#1}\\newline\\vspace*{1mm}}\n\\newcommand{\\ruler}{\\vspace*{3mm}\\noindent\\rule{\\columnwidth}{0.4pt}}\n\\newcommand{\\halfruler}{\\vspace*{3mm}\\noindent\\begin{center}\\rule{0.5\\columnwidth}{0.4pt}\\end{center}}\n\\renewcommand{\\vec}{\\mbox{\\rm vec}}\n\n\\makeatletter\n\\def\\mean{\\mathop{\\operator@font mean}}\n\\makeatother\n\n\\begin{document}\n');
         else;
            fprintf(fout,'\n\\clearpage\n\\newline\n');
         end;
         fprintf(fout,['\n\\section*{Structural VAR --- version ' num2str(myvers,'%0.2f') ' \\\\\n']);
         fprintf(fout,['{\\small Copyright \\copyright\\ Anders Warne, 2001-' datestr(now,10) '}}\n\n']);
      else;
         fprintf(fout,'\n');
         fprintf(fout,'*******************************************************************************\n');
         fprintf(fout,'\n');
         fprintf(fout,['               Structural VAR   ---   version ' num2str(myvers,'%0.2f') '\n']);
         fprintf(fout,['               Copyright  Anders Warne, 2001-' datestr(now,10) '\n']);
         fprintf(fout,'\n');
         fprintf(fout,'*******************************************************************************\n');
         fprintf(fout,'\n');
      end;
      %
      if curr_ini.texoutputformat>=2;
         fprintf(fout,'\\begin{tabular}{ll}\n');
         fprintf(fout,['Endogenous variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.evars)));
         fprintf(fout,['Estimation Sample: & ' samplefirst ' -- ' samplelast ' \\\\\n']);
         fprintf(fout,['Initial Sample: & ' initfirst ' -- ' initlast ' \\\\\n']);
         fprintf(fout,['Number of lags: & ' num2str(cm.p) ' for endogenous variables \\\\\n']);
         if cm.q1+cm.q0>0;
            fprintf(fout,['                & ' num2str(cm.pe) ' for exogenous variables \\\\\n']);
         end;
         fprintf(fout,['Number of observations: & ' num2str(size(cm.x0,1)-max(cm.p,cm.pe)) ' \\\\\n']);
         if cm.cointvalue==1;
            fprintf(fout,['Deterministic variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.deter)));
            if strcmp(cm.restvec,'')==0;
               fprintf(fout,[' & %s in cointegration space \\\\\n'],txt2ltxmath(strtrim(cm.restvec)));
            end;
         else;
            fprintf(fout,['Deterministic variables: & %s \\\\\n'],txt2ltxmath(strtrim([cm.deter ' ' cm.restvec])));
         end;
         if strcmp(cm.deterdel,'')==0;
            fprintf(fout,'Deleted variables: & %s \\\\\n',txt2ltxmath(strtrim(cm.deterdel)));
         end;
         if exo==1;
            if cm.q1>0;
               fprintf(fout,['Exogenous I(1) variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.xvars1)));
            end;
            if size(cm.x2,2)-cm.q1>0;
               fprintf(fout,['Exogenous I(0) variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.xvars0)));
            end;
         end;
         fprintf(fout,'\\end{tabular}\n\n\\ruler\n\n');
      else;
         fprintf(fout,['Endogenous variables:     ' cm.evars '\n']);
         fprintf(fout,['Estimation Sample:        ' samplefirst ' - ' samplelast '\n']);
         fprintf(fout,['Initial Sample:           ' initfirst ' - ' initlast '\n']);
         fprintf(fout,['Number of lags:           ' num2str(cm.p) ' for endogenous variables\n']);
         if cm.q1+cm.q0>0;
            fprintf(fout,['                          ' num2str(cm.pe) ' for exogenous variables\n']);
         end;
         fprintf(fout,['Number of observations:   ' num2str(size(cm.x0,1)-max(cm.p,cm.pe)) '\n']);
         if cm.cointvalue==1;
            fprintf(fout,['Deterministic variables:  ' cm.deter '\n']);
            if strcmp(cm.restvec,'')==0;
               fprintf(fout,['   ' cm.restvec ' in cointegration space\n']);
            end;
         else;
            fprintf(fout,['Deterministic variables:  ' strtrim([cm.deter ' ' cm.restvec]) '\n']);
         end;
         if strcmp(cm.deterdel,'')==0;
            fprintf(fout,'Deleted variables:        %s\n',cm.deterdel);
         end;
         if exo==1;
            if cm.q1>0;
               fprintf(fout,['Exogenous I(1) variables: ' cm.xvars1 '\n']);
            end;
            if size(cm.x2,2)-cm.q1>0;
               fprintf(fout,['Exogenous I(0) variables: ' cm.xvars0 '\n']);
            end;
         end;
         fprintf(fout,'\n*******************************************************************************\n\n');
      end;
      %
      if cm.cointvalue==1;
         if curr_ini.texoutputformat>=2;
            fprintf(fout,'\\subsection*{I(1) analysis:}\n\n');
            if cm.dummies==0;
               if cm.n<=12;
                  if cm.critvalnum==6;
                     if (cm.q1+cm.q0)==0;
                        mypvalues = tracepval(cm.n,cm.trace,cm.deterselect);
                     elseif (cm.q1>0)&(cm.q1<12)&(cm.q0==0)&((cm.deterselect(1,2)==1)|(cm.deterselect(1,4)==1));
                        mypvalues = tracepval_exo(cm.n,cm.q1,cm.trace,cm.deterselect);
                     elseif (cm.q1==0)&(cm.q0>0);
                        mypvalues = tracepval_exoi0(cm.n,cm.p,cm.pe,cm.q0,cm.trace,cm.deterselect,cm.d,cm.beta,cm.S_01,cm.M_02,cm.M_12,cm.M_22,cm.Z0,cm.Z1,cm.Z2);
                     else;
                        mypvalues = NaN;
                        for j=2:cm.n;
                           mypvalues = [mypvalues;NaN];
                        end;
                     end;
                     if isempty(cm.bartcorr)==1;
                        fprintf(fout,'\\begin{tabular}{rrrr}\n');
                        fprintf(fout,'Rank & Eigenvalues & $p$-values & Trace tests \\\\\n');
                        fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(0:cm.n-1)' cm.lambda mypvalues cm.trace]');
                     else;
                        fprintf(fout,'\\begin{tabular}{rrrrr}\n');
                        fprintf(fout,'Rank & Eigenvalues & $p$-values & Trace tests & Bartlett factor \\\\\n');
                        fprintf(fout,' & & & & (corrected) \\\\\n');
                        resmat =[(0:cm.n-1)' cm.lambda mypvalues cm.trace cm.bartcorr];
                        fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],resmat');
                     end;
                     fprintf(fout,'\\end{tabular}\n\n');
                  else;
                     if (cm.q1+cm.q0)==0;
                        mydist = tracedist(cm.n,cm.critvalnum,cm.deterselect);
                     elseif (cm.q1>0)&(cm.q1<12)&(cm.q0==0)&((cm.deterselect(1,2)==1)|(cm.deterselect(1,4)==1));
                        mydist = trace_exo_dist(cm.n,cm.q1,cm.deterselect,cm.critvalnum);
                     elseif (cm.q1==0)&(cm.q0>0);
                        mydist = trace_exoi0_dist(cm.n,cm.p,cm.pe,cm.q0,cm.critvalnum,cm.deterselect,cm.d,cm.beta,cm.S_01,cm.M_02,cm.M_12,cm.M_22,cm.Z0,cm.Z1,cm.Z2);
                     else;
                        mydist = NaN;
                        for j=2:cm.n;
                           mydist = [mydist;NaN];
                        end;
                     end;
                     if isempty(cm.bartcorr)==1;
                        fprintf(fout,'\\begin{tabular}{rrrr}\n');
                        mystr = ['Rank & Eigenvalues & ' num2str(100-curr_ini.critval) '\\%c quantile & Trace tests \\\\\n'];
                        fprintf(fout,mystr,'%');
                        fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(0:cm.n-1)' cm.lambda mydist cm.trace]');
                     else;
                        fprintf(fout,'\\begin{tabular}{rrrrr}\n');
                        mystr = ['Rank & Eigenvalues & ' num2str(100-curr_ini.critval) '\\%c quantile & Trace tests & Bartlett factor \\\\\n'];
                        fprintf(fout,mystr,'%');
                        fprintf(fout,' & & & & (corrected) \\\\\n');
                        resmat = [(0:cm.n-1)' cm.lambda mydist cm.trace cm.bartcorr];
                        fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],resmat');
                     end;
                     fprintf(fout,'\\end{tabular}\n\n');
                  end;
               else;
                  if isempty(cm.bartcorr)==1;
                     fprintf(fout,'\\begin{tabular}{rrr}\n');
                     fprintf(fout,'Rank & Eigenvalues & Trace tests \\\\\n');
                     fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(0:cm.n-1)' cm.lambda cm.trace]');
                  else;
                     fprintf(fout,'\\begin{tabular}{rrrr}\n');
                     fprintf(fout,'Rank & Eigenvalues & Trace tests & Bartlett factor \\\\\n');
                     fprintf(fout,' & & & (corrected)\n');
                     resmat = [(0:cm.n-1)' cm.lambda cm.trace cm.bartcorr];
                     fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],resmat');
                  end;
                  fprintf(fout,'\\end{tabular}\n\n');
               end;
            else;
               fprintf(fout,'\\begin{tabular}{rrr}\n');
               fprintf(fout,'Rank & Eigenvalues & Trace tests \\\\\n');
               fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(0:cm.n-1)' cm.lambda cm.trace]');
               fprintf(fout,'\\end{tabular}\n\n');
            end;
            fprintf(fout,'\\subsection*{Estimated cointegration relations:}\n\n');
            myprintstr = '%2.0f';
            r_str = 'r';
            j = 2;
            while j<=size(cm.beta,1)+1;
               myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
               r_str = [r_str 'r'];
               j = j+1;
            end;
            myprintstr = [myprintstr ' \\\\\n'];
            fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
            fprintf(fout,'relation & %s\n',text_latextable(strtrim([cm.evars ' ' cm.xvars1 ' ' cm.restvec])));
%            fprintf(fout,['relation & ' strrep(strtrim([txt2ltxmath(strtrim(cm.evars)) ' ' txt2ltxmath(strtrim(cm.xvars1)) ' ' txt2ltxmath(strtrim(cm.restvec))]),' ',' & ') ' \\\\\n']);
            fprintf(fout,myprintstr,[(1:cm.n)' normbeta(cm.beta)']');
            fprintf(fout,'\\end{tabular}\n\n');
         else;
            fprintf(fout,'I(1) analysis:\n\n');
            if cm.dummies==0;
               if cm.n<=12;
                  if cm.critvalnum==6;
                     if (cm.q1+cm.q0)==0;
                        mypvalues = tracepval(cm.n,cm.trace,cm.deterselect);
                     elseif (cm.q1>0)&(cm.q1<12)&(cm.q0==0)&((cm.deterselect(1,2)==1)|(cm.deterselect(1,4)==1));
                        mypvalues = tracepval_exo(cm.n,cm.q1,cm.trace,cm.deterselect);
                     elseif (cm.q1==0)&(cm.q0>0);
                        mypvalues = tracepval_exoi0(cm.n,cm.p,cm.pe,cm.q0,cm.trace,cm.deterselect,cm.d,cm.beta,cm.S_01,cm.M_02,cm.M_12,cm.M_22,cm.Z0,cm.Z1,cm.Z2);
                     else;
                        mypvalues = NaN;
                        for j=2:cm.n;
                           mypvalues = [mypvalues;NaN];
                        end;
                     end;
                     if isempty(cm.bartcorr)==1;
                        fprintf(fout,['Rank    Eigenvalues ' sp_str '  p-values  ' sp_str ' Trace tests\n\n']);
                        fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(0:cm.n-1)' cm.lambda mypvalues cm.trace]');
                     else;
                        fprintf(fout,['Rank    Eigenvalues  ' sp_str ' p-values  ' sp_str ' Trace tests      Bartlett factor\n']);
                        fprintf(fout,['                     ' sp_str '           ' sp_str ' (corrected)\n']);
                        fprintf(fout,'\n');
                        resmat =[(0:cm.n-1)' cm.lambda mypvalues cm.trace cm.bartcorr];
                        fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],resmat');
                     end;
                  else;
                     if (cm.q1+cm.q0)==0;
                        mydist = tracedist(cm.n,cm.critvalnum,cm.deterselect);
                     elseif (cm.q1>0)&(cm.q1<12)&(cm.q0==0)&((cm.deterselect(1,2)==1)|(cm.deterselect(1,4)==1));
                        mydist = trace_exo_dist(cm.n,cm.q1,cm.deterselect,cm.critvalnum);
                     elseif (cm.q1==0)&(cm.q0>0);
                        mydist = trace_exoi0_dist(cm.n,cm.p,cm.pe,cm.q0,cm.critvalnum,cm.deterselect,cm.d,cm.beta,cm.S_01,cm.M_02,cm.M_12,cm.M_22,cm.Z0,cm.Z1,cm.Z2);
                     else;
                        mydist = NaN;
                        for j=2:cm.n;
                           mydist = [mydist;NaN];
                        end;
                     end;
                     if isempty(cm.bartcorr)==1;
                        mystr = ['Rank    Eigenvalues ' sp_str num2str(100-curr_ini.critval,'%0.0f') '%c quantile ' sp_str 'Trace tests\n'];
                        fprintf(fout,mystr,'%');
                        fprintf(fout,'\n');
                        fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(0:cm.n-1)' cm.lambda mydist cm.trace]');
                     else;
                        mystr = ['Rank    Eigenvalues ' sp_str num2str(100-curr_ini.critval,'%0.0f') '%c quantile ' sp_str 'Trace tests   ' sp_str ' Bartlett factor\n'];
                        fprintf(fout,mystr,'%');
                        fprintf(fout,'                                     (corrected)\n');
                        fprintf(fout,'\n');
                        resmat = [(0:cm.n-1)' cm.lambda mydist cm.trace cm.bartcorr];
                        fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],resmat');
                     end;
                  end;
               else;
                  if isempty(cm.bartcorr)==1;
                     fprintf(fout,['Rank    Eigenvalues ' sp_str ' Trace tests\n']);
                     fprintf(fout,'\n');
                     fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f    %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(0:cm.n-1)' cm.lambda cm.trace]');
                  else;
                     fprintf(fout,['Rank    Eigenvalues ' sp_str ' Trace tests   ' sp_str ' Bartlett factor\n']);
                     fprintf(fout,['                    ' sp_str ' (corrected)\n']);
                     fprintf(fout,'\n');
                     resmat = [(0:cm.n-1)' cm.lambda cm.trace cm.bartcorr];
                     fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f    %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],resmat');
                  end;
               end;
            else;
               fprintf(fout,['Rank    Eigenvalues ' sp_str ' Trace tests\n\n']);
               fprintf(fout,['%2.0f   %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f    %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(0:cm.n-1)' cm.lambda cm.trace]');
            end;
            fprintf(fout,'\n');
            fprintf(fout,'Estimated cointegration relations:\n');
            fprintf(fout,'\n');
            myprintstr = '%2.0f';
            j = 2;
            while j<=size(cm.beta,1)+1;
               myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
               j = j+1;
            end;
            myprintstr = [myprintstr '\n'];
            fprintf(fout,myprintstr,[(1:cm.n)' normbeta(cm.beta)']');
            fprintf(fout,'\n');
         end;
      end;
      %
      if curr_ini.texoutputformat>=2;
         fprintf(fout,'\\subsection*{Properties of the stacked VAR(1) companion matrix:}\n\n');
         fprintf(fout,'\\begin{tabular}{rrrr}\n');
         fprintf(fout,'\\multicolumn{4}{c}{Eigenvalues} \\\\\n');
         fprintf(fout,'number & real & imaginary & modulus \\\\\n');
         fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(1:size(eigenvalues,1))' real(eigenvalues) imag(eigenvalues) abs(eigenvalues)]');
         fprintf(fout,'\\end{tabular}\n\n');
      else;
         fprintf(fout,'Properties of the stacked VAR(1) companion matrix:\n');
         fprintf(fout,'Eigenvalues\n');
         fprintf(fout,['number    real  ' sp_str '    imaginary  ' sp_str '   modulus\n']);
         fprintf(fout,'\n');
         fprintf(fout,['%3.0f      %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f     %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f         %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(1:size(eigenvalues,1))' real(eigenvalues) imag(eigenvalues) abs(eigenvalues)]');
         fprintf(fout,'\n');
      end;
      %
      if (curr_ini.shortdyn==1);
         txt = strvcat(' ','Initializing...','Computing short-run dynamics.','Please hold!');
         setlogtext('Computing short-run dynamics...',maingui);
         set(waitf.text,'String',txt);
         if curr_ini.texoutputformat>=2;
            fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
         else;
            fprintf(fout,'===============================================================================\n\n');
         end;
         fclose(fout);
         shortstr = shortrun(cm.alpha,cm.Psi,cm.Omega,cm.beta,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,cm.cointvalue,cm.outputfile,0);
         fout = fopen(cm.outputfile,'at');
         if curr_ini.texoutputformat>=2;
            fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
         else;
            fprintf(fout,'\n===============================================================================\n\n');
         end;
         if (cm.r<cm.n+cm.q1)&~((cm.p==1)&(cm.r==0));
            C = cmat_stat(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1);
            myC = showc(C(:,1:cm.n),cm.myvars);
            myCstd = showc(C(:,cm.n+1:2*cm.n),cm.myvars);
            myCt = showc(inf2nan(C(:,1:cm.n)./C(:,cm.n+1:2*cm.n)),cm.myvars);
            if curr_ini.texoutputformat>=2;
               myC = text_latextable(myC);
               myCstd = text_latextable(myCstd);
               myCstd = myCstd(2:size(myCstd,1),:);
               myCt = text_latextable(myCt);
               myCt = myCt(2:size(myCt,1),:);
               myC = merge_latextable(myC,myCstd,myCt);
               r_str = 'r';
               for i=1:cm.n;
                  r_str = [r_str 'r'];
               end;
               fprintf(fout,'\\subsection*{The $C$ matrix}\n\n\\header{For $r$=%0.0f}\n\n',cm.n);
               fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
               for i=1:(size(myC,1));
                  fprintf(fout,'%s\n',myC(i,:));
               end;
               fprintf(fout,'\\end{tabular}\n\n');
               fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
            else;
               fprintf(fout,'THE C-MATRIX (for rank=%1.0f)\n\n',cm.n);
               for i=1:(size(myC,1));
                  fprintf(fout,'%s\n',myC(i,:));
               end;
               fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
               for i=1:(size(myC,1));
                  fprintf(fout,'%s\n',myCstd(i,:));
               end;
               fprintf(fout,'\nt-ratios:\n\n');
               for i=1:(size(myC,1));
                  fprintf(fout,'%s\n',myCt(i,:));
               end;
               fprintf(fout,'\n===============================================================================\n\n');
            end;
         end;
      end;
      %
      % Calculate specification tests
      %
      [lnl,ldOmega] = loglike(cm.epsilon);
      if curr_ini.texoutputformat>=2;
         fprintf(fout,'\\subsection*{Value of the maximized log-likelihood function:}\n\n');
         fprintf(fout,'\\begin{tabular}{ll}\n');
         fprintf(fout,[' & $\\ln L(\\mbox{\\rm lags}=%1.0f,r=%1.0f)$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.p,cm.n,lnl);
         fprintf(fout,[' & $\\ln (\\det(\\Omega))$ = %1.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],ldOmega);
         fprintf(fout,'\\end{tabular}\n\n\\ruler\n\n%%==============================================================================\n\n');
      else;
         fprintf(fout,'Value of the maximized log-likelihood function:\n\n');
         fprintf(fout,['   lnL(lags=%1.0f,rank=%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],cm.p,cm.n,lnl);
         fprintf(fout,['   ln(det(Omega)) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],ldOmega);
         fprintf(fout,'\n===============================================================================\n\n');
      end;
      %
      % tests on deterministics
      %
      if cm.cointvalue==1;
         if cm.deterselect(1,1)==1;
            test_type = 'uconst';
            setlogtext('Testing unrestricted vs. restricted constant model...');
         elseif cm.deterselect(1,2)==1;
            test_type = 'rconst';
            setlogtext('Testing restricted vs. zero constant model...');
         elseif cm.deterselect(1,3)==1;
            test_type = 'utrend';
            setlogtext('Testing unrestricted vs. restricted linear trend model...');
         elseif cm.deterselect(1,4)==1;
            test_type = 'rtrend';
            setlogtext('Testing restricted vs. zero linear trend model...');
         end;
         %
         lr_tests = [];
         d_fs = [];
         p_vals = [];
         my_ranks = [];
         for i=1:cm.n-1;
            %
            % create a new epsilon series
            %
            beta_n = cm.beta(:,1:i);
            alpha_n = (cm.S_01*beta_n)*inverse(beta_n'*cm.S_11*beta_n);
            if isempty(cm.Z2)==0;
               Psi_n = (cm.M_02*inverse(cm.M_22))-(alpha_n*beta_n'*cm.M_12*inverse(cm.M_22));
               epsilon_n = cm.Z0-(alpha_n*beta_n'*cm.Z1)-(Psi_n*cm.Z2);
            else;
               epsilon_n = cm.Z0-(alpha_n*beta_n'*cm.Z1);
            end;
            %
            [lr_test,d_f,p_val] = rest_const_trend(test_type,epsilon_n,i,cm.Z0,cm.Z1,cm.Z2,cm.df,curr_ini.sscorrect);
            %
            % collect results
            %
            lr_tests = [lr_test;lr_tests];
            d_fs = [d_f;d_fs];
            p_vals = [p_val;p_vals;];
            my_ranks = [i;my_ranks];
         end;
         %
         % print the results
         %
         if curr_ini.texoutputformat>=2;
            if cm.deterselect(1,1)==1;
               txt_str = '\subsection*{Testing Unrestricted vs. Restricted Constant Model}';
               my_txt_null = '\delta_{0} = \alpha\mu_{0}';
               my_txt_alt = '\delta_{0} \neq \alpha\mu_{0}';
            elseif cm.deterselect(1,2)==1;
               txt_str = '\subsection*{Testing Restricted vs. Zero Constant Model}';
               my_txt_null = '\mu_{0} = 0';
               my_txt_alt = '\mu_{0} \neq 0';
            elseif cm.deterselect(1,3)==1;
               txt_str = '\subsection*{Testing Unrestricted vs. Restricted Linear Trend Model}';
               my_txt_null = '\delta_{1} = \alpha\mu_{1}';
               my_txt_alt = '\delta_{1} \neq \alpha\mu_{1}';
            elseif cm.deterselect(1,4)==1;
               txt_str = '\subsection*{Testing Restricted vs. Zero Linear Trend Model}';
               my_txt_null = '\mu_{1} = 0';
               my_txt_alt = '\mu_{1} \neq 0';
            end;
            fprintf(fout,'%s\n',txt_str);
            fprintf(fout,'\\header{LR-type test statistic}\n\n');
            fprintf(fout,'\\begin{tabular}{rrrr}\n');
            fprintf(fout,'$H_{0}$: & \\multicolumn{3}{l}{$%s$} \\\\\n',my_txt_null);
            fprintf(fout,'$H_{A}$: & \\multicolumn{3}{l}{$%s$} \\\\\n',my_txt_alt);
            if curr_ini.sscorrect==0;
               fprintf(fout,'Rank & LR & df & $p$-value \\\\\n');
               prt_str = ['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'];
            else;
               fprintf(fout,'Rank & F & df & $p$-value \\\\\n');
               prt_str = ['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & (%0.0f,%0.0f) & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'];
            end;
            fprintf(fout,prt_str,[my_ranks lr_tests d_fs p_vals]');
            fprintf(fout,'\\end{tabular}\n\n\\ruler\n\n%%==============================================================================\n\n');
         else;
            if cm.deterselect(1,1)==1;
               txt_str = 'TESTING UNRESTRICTED VS. RESTRICTED CONSTANT MODEL\n\n';
               my_txt_null = 'delta(0) = alpha*mu(0)';
               my_txt_alt = 'delta(0) not equal to alpha*mu(0)';
            elseif cm.deterselect(1,2)==1;
               txt_str = 'TESTING RESTRICTED VS. ZERO CONSTANT MODEL\n\n';
               my_txt_null = 'mu(0) = 0';
               my_txt_alt = 'mu(0) not equal to 0';
            elseif cm.deterselect(1,3)==1;
               txt_str = 'TESTING UNRESTRICTED VS. RESTRICTED LINEAR TREND MODEL\n\n';
               my_txt_null = 'delta(1) = alpha*mu(1)';
               my_txt_alt = 'delta(1) not equal to alpha*mu(1)';
            elseif cm.deterselect(1,4)==1;
               txt_str = 'TESTING RESTRICTED VS. ZERO LINEAR TREND MODEL\n\n';
               my_txt_null = 'mu(1) = 0';
               my_txt_alt = 'mu(1) not equal to 0';
            end;
            fprintf(fout,txt_str);
            fprintf(fout,'  LR-type test statistic\n  ~~~~~~~~~~~~~~~~~~~~~~\n');
            fprintf(fout,'Null Hypothesis:        %s\n',my_txt_null);
            fprintf(fout,'Alternative Hypothesis: %s\n\n',my_txt_alt);
            if curr_ini.sscorrect==0;
               fprintf(fout,['Rank      LR  ' sp_str ' df     p-value \n']);
               prt_str = [' %2.0f   %' num2str(4+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   %2.0f     %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'];
            else;
               fprintf(fout,['Rank      F   ' sp_str '    df         p-value \n']);
               prt_str = [' %2.0f   %' num2str(4+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f   (%2.0f,%3.0f)      %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'];
            end;
            fprintf(fout,prt_str,[my_ranks lr_tests d_fs p_vals]');
            fprintf(fout,'\n===============================================================================\n\n');
         end;
      end;
      %
      fclose(fout);
      if (curr_ini.excludestat==1)&(cm.q1>0)&(cm.cointvalue==1);
         setlogtext('Computing exclusion tests...');
         txt = strvcat(' ','Initializing...','Computing exclusion restrictions tests.','Please hold!');
         set(waitf.text,'String',txt);
         print_exclusion(cm,curr_ini);
         setlogtext('Computing stationarity tests...');
         txt = strvcat(' ','Initializing...','Computing stationarity tests.','Please hold!');
         set(waitf.text,'String',txt);
         print_stationarity(cm,curr_ini,0);
      end;
      df = cm.n*cm.p+cm.q1*(cm.pe+1)+cm.q0*(cm.pe+1)+cm.d+cm.dcoint;
      %
      % compute tests for additional deterministic variables
      %
      if (curr_ini.dummytests==1)&(cm.datad>0);
         txt = strvcat(' ','Initializing...','Computing exclusion tests for additonal deterministics.','Please hold!');
         set(waitf.text,'String',txt);
         print_exclude_dummies(cm,curr_ini);
      end;
      %
      % Calculate lag-order test
      %
      if curr_ini.lagtest>0;
         txt = strvcat(' ','Initializing...','Computing lag order tests.','Please hold!');
         setlogtext('Computing lag order tests...',maingui);
         set(waitf.text,'String',txt);
         print_lagordertest(cm,curr_ini,cm.beta,cm.alpha,cm.Psi,cm.epsilon,[],0,0,cm.T-df,0);
      end;
      if curr_ini.gctest==1;
         txt = strvcat(' ','Initializing...','Computing Granger causality tests.','Please hold!');
         setlogtext('Computing Granger causality tests...',maingui);
         set(waitf.text,'String',txt);
         print_gctest(cm,curr_ini,cm.beta,cm.alpha,cm.Psi,cm.epsilon,[],0,0,cm.T-df,0);
         if cm.n>2;
            txt = strvcat(' ','Initializing...','Computing multi-step Granger causality tests.','Please hold!');
            setlogtext('Computing multi-step Granger causality tests...',maingui);
            set(waitf.text,'String',txt);
            print_mgctest(cm,curr_ini,cm.beta,cm.alpha,cm.Psi,cm.epsilon,[],0,0,0,0);
         end;
      end;
      if curr_ini.excludediffs==1;
         if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
            txt = strvcat(' ','Initializing...','Computing exclusion tests for first differences.','Please hold!');
            setlogtext('Computing exclusion tests for first differences...',maingui);
            set(waitf.text,'String',txt);
            print_firstdiffstests(cm,curr_ini,cm.beta,cm.alpha,cm.Psi,cm.epsilon,[],0,0,df,0);
         end;
      end;
      %
      % Compute statistics for the residuals if so desired
      %
      print_residual_analysis(cm,curr_ini,cm.epsilon,0,0,df);
      %
      % specification tests
      %
      if round(sum(cm.spectest))>0;
         txt = strvcat(' ','Initializing...','Computing specification tests.','Please hold!');
         set(waitf.text,'String',txt);
      end;
      if cm.spectest(1,1)==1;
         setlogtext('Computing serial correlation tests...',maingui);
         print_corrtest(cm,curr_ini,cm.epsilon,cm.beta,0,0,df);
      end;
      %
      if cm.spectest(1,2)==1;
         setlogtext('Computing ARCH tests...',maingui);
         print_archtest(cm,curr_ini,cm.epsilon,0,0,df);
      end;
      %
      if cm.spectest(1,3)==1;
         setlogtext('Computing normality tests...',maingui);
         print_normalitytest(cm,curr_ini,cm.epsilon,0,0,df);
      end;
      %
      if cm.spectest(1,4)==1;
         if cm.deterselect(1,3)==1;
            cm.trendplace = 2;
         else;
            cm.trendplace = 0;
         end;
         %
         fout = fopen(cm.outputfile,'at');
         if curr_ini.texoutputformat>=2;
            fprintf(fout,['\\subsection*{Parameter constancy tests}\n\n\\header{For $r$=' num2str(cm.r) ', lags=' num2str(cm.p) ', \\#free $\\beta$ params=' num2str(cm.r*(cm.n-cm.r),'%0.0f') ', \\#free $\\alpha$ params=' num2str(cm.n*cm.r,'%0.0f') '}\n\n']);
         else;
            fprintf(fout,['PARAMETER CONSTANCY TESTS\n\n(for rank=' num2str(cm.r) ',lags=' num2str(cm.p) ',#free beta params=' num2str(cm.r*(cm.n-cm.r),'%0.0f') ',#free alpha params=' num2str(cm.n*cm.r,'%0.0f') ')\n\n']);
         end;
         fclose(fout);
         if curr_ini.lstrtest==1;
            setlogtext('Computing LSTR1 constancy tests...',maingui);
            print_lstr1test(cm,curr_ini,cm.epsilon,cm.beta'*cm.Z1,df);
         end;
         %
         if cm.evalue<=1;
            setlogtext('Computing Hansen-Johansen constancy tests...',maingui);
            print_jo_hansen(cm,curr_ini,df);
         end;
         %
         if curr_ini.chow==1;
            setlogtext('Computing Chow tests...',maingui);
            print_chow(cm,curr_ini,cm.beta,0);
         end;
         %
         setlogtext('Computing fluctuation tests...',maingui);
         print_ploberger(cm,curr_ini,cm.beta,0);
      end;
   end;
   %
   % Close wait dialogue
   %
   set(waitf.text,'UserData','done');
   delete(mywait);
   drawnow;
   %
   % Display results from ML cointegration analysis
   %
   if cm.modeltype(1,1)==1;
      my_loop_svar = 1;
   elseif cm.modeltype(1,2)==1;
      my_loop_svar = 0;
   end;
   my_quitting = '';
   myloopvar = 1;
   first_run = 1;
   run_second = 0;
   while myloopvar==1;
      if (cm.cointvalue==1)&(first_run==1);
         if isempty(cm.Z2)==0;
            cm.R1t = cm.Z1t-((cm.M_12/cm.M_22)*cm.Z2);
         else;
            cm.R1t = cm.Z1t;
         end;
         cm.beta = savebeta;
         if run_second==1;
            %
            % get the old values of alpha, Psi, epsilon, and Omega
            %
            cm.alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
            if isempty(cm.Z2)==0;
               cm.Psi = (cm.M_02/cm.M_22)-(cm.alpha*cm.beta'*(cm.M_12/cm.M_22));
               cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1)-(cm.Psi*cm.Z2);
            else;
               cm.Psi = [];
               cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1);
            end;
            cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
         end;
         rankdlg = rankgui('init',cm);
         drawnow;
         coint = get(rankdlg,'UserData');
         cm.r = get(coint.mypopup,'Value')-1;
         %
         c_ini = get(coint.viewdata,'UserData');
         c_m = c_ini.model;
         cm.evalue = c_m.evalue;
         %
         thisexit = get(coint.mypopup,'UserData');
         set(coint.eigenvalue,'UserData','done');
         delete(rankdlg);
         drawnow;
         if thisexit==2;
            %
            % Close the outputfile
            %
            if curr_ini.noprint==0;
               fout = fopen(cm.outputfile,'at');
               if curr_ini.texoutputformat>=2;
                  fprintf(fout,['\\header{Output created on: ' datestr(now,0) '}\n']);
               else;
                  fprintf(fout,['Output created on: ' datestr(now,0) '\n']);
               end;
               fclose(fout);
            end;
            setlogtext('Estimation aborted...',maingui);
            return;
         else;
            %
            %
            %
            dfc = round(((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)/cm.n);
            cm.df = cm.n*(cm.p-1)+cm.r+cm.q1*cm.pe+cm.q0*(cm.pe+1)+cm.d+dfc;
         end;
         setlogtext(['You selected ' num2str(cm.r) ' cointegration vectors...'],maingui);
         eigenvalues = companion(cm.Z0,cm.Z1,cm.Z2,cm.r,cm.p,cm.n,cm.q1,cm.q0,cm.d,cm.T);
         [eigenvalues2,AscInd] = sort(abs(eigenvalues));
         nr = size(AscInd,1);
         DescInd = zeros(nr,1);
         i = 1;
         while i<=nr;
            DescInd(i)=AscInd(nr+1-i);
            i =i+1;
         end;
         if curr_ini.noprint==0;
            fout = fopen(cm.outputfile,'at');
            eigenvalues = eigenvalues(DescInd);
            if curr_ini.texoutputformat>=2;
               fprintf(fout,['\\header{You selected ' num2str(cm.r) ' cointegration vectors}\n\n']);
               if ((cm.r>0)&(cm.r<cm.n))==1;
                  fprintf(fout,'\\subsection*{Properties of the stacked VAR(1) companion matrix:}\n\n');
                  fprintf(fout,'\\begin{tabular}{rrrr}\n');
                  fprintf(fout,'\\multicolumn{4}{c}{Eigenvalues} \\\\\n');
                  fprintf(fout,'number & real & imaginary & modulus \\\\\n');
                  fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(1:size(eigenvalues,1))' real(eigenvalues) imag(eigenvalues) abs(eigenvalues)]');
                  fprintf(fout,'\\end{tabular}\n\n');
               end;
               fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
            else;
               fprintf(fout,['You selected ' num2str(cm.r) ' cointegration vectors\n\n']);
               if ((cm.r>0)&(cm.r<cm.n))==1;
                  fprintf(fout,'Properties of the stacked VAR(1) companion matrix:\n');
                  fprintf(fout,'Eigenvalues\n');
                  fprintf(fout,['number    real ' sp_str '     imaginary  ' sp_str '   modulus\n']);
                  fprintf(fout,'\n');
                  fprintf(fout,['%3.0f      %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f     %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f         %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(1:size(eigenvalues,1))' real(eigenvalues) imag(eigenvalues) abs(eigenvalues)]');
                  fprintf(fout,'\n');
               end;
               fprintf(fout,'*******************************************************************************\n\n');
            end;
            fclose(fout);
         end;
      elseif cm.cointvalue==0;
         cm.r = cm.n;
      end;
      first_run = 0;
      %
      % Determining whether to estimate cm.beta under linear
      % restrictions or not
      %
      if cm.r==0;
         cm.alpha = 0;
         cm.beta = 0;
         cm.oldbeta = 0;
         if isempty(cm.Z2)==0;
            cm.Psi = cm.M_02/cm.M_22;
            cm.epsilon = cm.Z0-(cm.Psi*cm.Z2);
         else;
            cm.Psi = [];
            cm.epsilon = cm.Z0;
         end;
         cm.Theta = cm.Psi;
         cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
         inv_Z2 = eye(size(cm.Z2,1))/(cm.Z2*cm.Z2');
         cm.Sigma_theta = kron(inv_Z2,cm.Omega);
         myloopvar = 0;
      elseif ((cm.r==cm.n+cm.q1)|(cm.cointvalue==0));
         cm.alpha = (cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta);
         if isempty(cm.Z2)==0;
            cm.Psi = (cm.M_02/cm.M_22)-(cm.alpha*cm.beta'*(cm.M_12/cm.M_22));
            cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1)-(cm.Psi*cm.Z2);
            cm.Z3 = [cm.Z2;cm.Z1];
         else;
            cm.Psi = [];
            cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1);
            cm.Z3 = cm.Z1;
         end;
         cm.Theta = [cm.Psi cm.alpha*cm.beta'];
         cm.Omega = cm.S_00-(((cm.S_01*cm.beta)/(cm.beta'*cm.S_11*cm.beta))*cm.beta'*cm.S_10);
         inv_Z3 = eye(size(cm.Z3,1))/(cm.Z3*cm.Z3');
         cm.Sigma_theta = kron(inv_Z3,cm.Omega);
         myloopvar = 0;
      else;
         cm.beta = cm.beta(:,1:cm.r);
         cm.oldbeta = cm.beta;
         %
         % Display estimated cm.beta and construct restriction matrices
         %
         hbeta = betagui('init',cm);
         drawnow;
         cointbeta = get(hbeta,'UserData');
         %
         % learn about degrees of freedom
         %
         c_ini = get(cointbeta.viewdata,'UserData');
         c_m = c_ini.model;
         cm.df = c_m.df;
         cm.dfbeta = c_m.dfbeta;
         cm.restrict = c_m.restrict;
         if isfield(c_m,'dfalpha')==1;
            if cm.restrict(1,1)==1;
               cm.dfalpha = c_m.dfalpha;
            end;
         end;
         if isfield(c_m,'LRbeta')==1;
            cm.LRbeta = c_m.LRbeta;
         else;
            cm.LRbeta = [];
         end;
         if isfield(c_m,'LRalpha')==1;
            cm.LRalpha = c_m.LRalpha;
         else;
            cm.LRalpha = 0;
         end;
         cm.evalue = c_m.evalue;
         %
         thisexit = get(cointbeta.clear,'UserData');
         betarest = get(cointbeta.betamat,'UserData');
         if my_loop_svar==1;
            my_quitting = get(cointbeta.stop,'UserData');
         else;
            my_quitting = 'reduced';
         end;
         stdbetarest = get(cointbeta.model_info,'UserData');
         cm.Theta = get(cointbeta.estalpha,'UserData');
         cm.Sigma_theta = get(cointbeta.genalpha,'UserData');
         g2 = size(cm.Theta,2);
         if g2>cm.r;
            cm.Psi = cm.Theta(:,1:g2-cm.r);
         else;
            cm.Psi = [];
         end;
         cm.alpha = cm.Theta(:,g2-cm.r+1:g2);
         set(cointbeta.graphs,'UserData','done');
         delete(hbeta);
         drawnow;
         if thisexit==1;
            myloopvar = 0;
            if strcmp(my_quitting,'quit')==1;
               my_loop_svar = 0;
            end;
         else;
            first_run = 1;
         end;
         %
         % if restrict=1 use restricted cm.beta estimate
         %
         if cm.restrict(1,2)==1;
            if strcmp(my_quitting,'quit')==1;
               setlogtext(strvcat(' ','Aborting estimation...'),maingui);
            else;
               setlogtext(strvcat(' ',['You selected ' num2str(cm.r) ' restricted cointegration vectors...']),maingui);
            end;
            cm.beta = betarest;
            cm.stdbeta = stdbetarest;
         else;
            if myloopvar==0;
               if strcmp(my_quitting,'quit')==1;
                  setlogtext(strvcat(' ','Aborting estimation...'),maingui);
               else;
                  setlogtext(strvcat(' ',['You selected ' num2str(cm.r) ' cointegration vectors...']),maingui);
               end;
            else;
               setlogtext(strvcat(' ','Returning to cointegration rank selection...'),maingui);
               run_second = 1;
            end;
         end;
         %
         % Estimating cm.Omega, cm.epsilon
         %
         if isempty(cm.Z2)==0;
            cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1)-(cm.Psi*cm.Z2);
         else;
            cm.epsilon = cm.Z0-(cm.alpha*cm.beta'*cm.Z1);
         end;
         cm.Omega = (1/cm.T)*cm.epsilon*cm.epsilon';
      end;
      %
      % structural analysis
      %
      if (first_run==0)&(my_loop_svar==1);
         svar_handle = svargui('init',cm);
         svar = get(svar_handle,'UserData');
         thisexit = get(svar.ploberger,'UserData');
         set(svar.residgraph,'UserData','done');
         delete(svar_handle);
         drawnow;
         if ispc==1;
            file = [pwd '\structural_est.txt'];
         else;
            file = [pwd '/structural_est.txt'];
         end;
         if fileexist(file)==1;
            delete_file_fcn(file);
         end;
         if thisexit==1;
            myloopvar = 0;
         else;
            myloopvar = 1;
            if (cm.cointvalue==1)&((cm.r==0)|(cm.r==cm.n));
               first_run = 1;
            end;
            cm.restrict = zeros(1,2);
            cm.beta = cm.oldbeta;
            cm.stdbeta = [];
         end;
      end;
   end;
   if strcmp(my_quitting,'quit')==0;
      setlogtext(strvcat(' ','Estimation completed!'),maingui);
   end;
   if curr_ini.noprint==0;
      %
      % Close the outputfile
      %
      fout = fopen(cm.outputfile,'at');
      if curr_ini.texoutputformat>=2;
         if (cm.cointvalue==0)&(my_loop_svar~=1);
            fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
         end;
         fprintf(fout,['\\header{Output created on: ' datestr(now,0) '}\n']);
      else;
         if (cm.cointvalue==0)&(my_loop_svar~=1);
            fprintf(fout,'\n*******************************************************************************\n\n');
         end;
         fprintf(fout,['Output created on: ' datestr(now,0) '\n']);
      end;
      fclose(fout);
   end;
end;

%
% if curr_ini.method is Bayesian with Gaussian residuals
%

if curr_ini.method(1,3)==1;
   if cm.cointvalue==0;
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
      txt = 'The model is based on no cointegration estimation, i.e., a VAR model is levels. Currently, Structural VAR does not incorporate the necessary functionality for Bayesian analysis of this model.';
      about(txt,'error','Bad Model...',120,400);
      return;
   end;
   if (cm.q1>0)&(cm.cointvalue==1);
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
      txt = ['The model has ' num2str(cm.q1) ' exogenous I(1) variables. The Bayesian analysis in Structural VAR cannot, at this stage, incorporate such variables in the cointegration analysis.'];
      about(txt,'error','Bad Model...',120,400);
      return;
   end;
   if (cm.dcoint>0)&(cm.cointvalue==1);
      set(waitf.text,'UserData','done');
      delete(mywait);
      drawnow;
      txt = ['The model has ' num2str(cm.dcoint) ' deterministic variable restricted to the cointegration space. Currently, the Bayesian analysis in Structural VAR does not allow for such restrictions in the cointegration analysis.'];
      about(txt,'error','Bad Model...',120,400);
      return;
   end;
   %
   % Adjust cm.samplebeg to fit with first observation
   %
   cm.oldsamplebeg = cm.samplebeg;
   if strcmp(cm.freq,'m')==1;
      cm.samplebeg = cm.samplebeg+(max(cm.p,cm.pe)/12);
   elseif strcmp(cm.freq,'q')==1;
      cm.samplebeg = cm.samplebeg+(max(cm.p,cm.pe)/4);
   else;
      cm.samplebeg = cm.samplebeg+max(cm.p,cm.pe);
   end;
   lag_str = 'lags';
   if cm.p==1;
      lag_str = 'lag';
   end;
   [initfirst,initlast,samplefirst,samplelast] = sampledates(cm.oldsamplebeg,cm.sampleend,max(cm.p,cm.pe),cm.freq);
   if cm.q1+cm.q0>0;
      add_lag_str = ['Using ' num2str(cm.pe) ' lag'];
      if cm.pe~=1;
         add_lag_str = [add_lag_str 's'];
      end;
      add_lag_str = [add_lag_str ' for exogenous variables...'];
   else;
      add_lag_str = '';
   end;
   setlogtext(strvcat('Initializing Structural VAR Bayesian analysis...',' ',['Estimation sample: ' samplefirst ' - ' samplelast '...'],['Initial sample:    ' initfirst ' - ' initlast '...'],['Using ' num2str(cm.p,'%0.0f') ' ' lag_str ' for endogenous variables...'],add_lag_str,' '),maingui);
   if cm.cointvalue==1;
      txt = strvcat(' ','Initializing...','Computing fractional marginal likelihoods.','Please hold!');
      set(waitf.text,'String',txt);
      setlogtext('Computing fractional marginal likelihoods...',maingui);
   end;
   [cm.S_00,cm.S_01,cm.S_10,cm.S_11,cm.M_02,cm.M_22,cm.M_12,cm.beta,cm.lambda,cm.trace] = mlcointbasic(cm.Z0,cm.Z1,cm.Z2,cm.n);
   %
   % compute the fractional marginal likelihoods as discussed by Corrander and Villani (2004)
   %
   cm.log_fracs = zeros(cm.n+1,1);
   for i=0:cm.n;
      if i>0;
         cm.rank(i+1).beta = cm.beta(:,1:i);
         cm.rank(i+1).alpha = (cm.S_01*cm.rank(i+1).beta)/(cm.rank(i+1).beta'*cm.S_11*cm.rank(i+1).beta);
         if isempty(cm.Z2)==0;
            cm.rank(i+1).Psi = (cm.M_02/cm.M_22)-(cm.rank(i+1).alpha*cm.rank(i+1).beta'*(cm.M_12/cm.M_22));
            cm.rank(i+1).epsilon = cm.Z0-(cm.rank(i+1).alpha*cm.rank(i+1).beta'*cm.Z1)-(cm.rank(i+1).Psi*cm.Z2);
         else;
            cm.rank(i+1).Psi = [];
            cm.rank(i+1).epsilon = cm.Z0-(cm.rank(i+1).alpha*cm.rank(i+1).beta'*cm.Z1);
         end;
         c_r = size(cm.Z2,1)+i+round((size(cm.Z1,1)-i)*(i/cm.n));
      else;
         cm.rank(1).beta = [];
         cm.rank(1).alpha = [];
         if isempty(cm.Z2)==0;
            cm.rank(1).Psi = cm.M_02/cm.M_22;
            cm.rank(1).epsilon = cm.Z0-(cm.rank(i+1).Psi*cm.Z2);
         else;
            cm.rank(1).Psi = [];
            cm.rank(1).epsilon = cm.Z0;
         end;
         c_r = size(cm.Z2,1);
      end;
      m = cm.n+size(cm.Z1,1)+size(cm.Z2,1);
      cm.rank(i+1).Omega = (1/cm.T)*cm.rank(i+1).epsilon*cm.rank(i+1).epsilon';
      cm.rank(i+1).part_canon_corr = det(cm.rank(i+1).Omega)/det(cm.S_00);
      %
      % compute the product of the gamma function ratios in logs
      %
      cm.rank(i+1).log_gamma = gamma_product(cm.n,cm.T-c_r,m-c_r);
      cm.rank(i+1).log_fraction_marglike = -((cm.T-m)/2)*log(cm.rank(i+1).part_canon_corr)+cm.rank(i+1).log_gamma;
      cm.log_fracs(i+1,1) = cm.rank(i+1).log_fraction_marglike;
   end;
   %
   % we shall now normalize the log fractional marginal likelihoods by subtracting a common constant.
   % This constant is given by the lowest log fractional marginal likelihood. This constant is used to
   % handle units for the exponential function
   %
   sum_fml = 0;
   frac_marg_like = zeros(cm.n+1,1);
   for i=0:cm.n;
      frac_marg_like(i+1,1) = cm.rank(i+1).log_fraction_marglike;
      cm.rank(i+1).norm_fraction_marglike = fix_exp(cm.rank(i+1).log_fraction_marglike - mean(cm.log_fracs));
      sum_fml = sum_fml + cm.rank(i+1).norm_fraction_marglike;
   end;
   cm.rankprob = zeros(cm.n+1,1);
   for i=0:cm.n;
      cm.rankprob(i+1,1) = cm.rank(i+1).norm_fraction_marglike/sum_fml;
   end;
   %
   % Prepare scaling factors for endogenous and exogenous I(1) and I(0) variables
   % The scaling factors are in terms of variances
   %
   cm.scale_n = diag(cm.rank(cm.n+1).Omega);
   cm.scale_q1 = [];
   cm.scale_q0 = [];
   if cm.q1+cm.q0>0;
      y = cm.x2(cm.p+1:size(cm.x2,1),:)';
      if isempty(cm.old_x1_r)==1;
         x = [cm.x1(cm.p+1:size(cm.x2,1),:)]';
      else;
         x = [cm.x1(cm.p+1:size(cm.x2,1),:) cm.old_x1_r(cm.p+1:size(cm.x2,1),:)]';
      end;
      for i=1:cm.p;
         x = [x;cm.x2(cm.p+1-i:size(cm.x2,1)-i,:)'];
      end;
      b = y*x'*inverse(x*x');
      res = y-(b*x);
      sigma = (1/size(y,2))*res*res';
      s_f = diag(sigma);
      if cm.q1>0;
         cm.scale_q1 = s_f(1:cm.q1,1);
      end;
      if cm.q0>0;
         cm.scale_q0 = s_f(1+cm.q1:cm.q0+cm.q1,1);
      end;
   end;
   %
   % Write stuff to the output file
   %
   if curr_ini.noprint==0;
      if curr_ini.append==1;
         fileformat = 'at';
      else;
         fileformat = 'wt';
      end;
      fout = fopen(cm.outputfile,fileformat);
      if curr_ini.texoutputformat>=2;
         if strcmp(fileformat,'wt')==1;
            fprintf(fout,'\n');
            for i=1:size(curr_ini.texheadertxt,1);
               fprintf(fout,[strrep(strtrim(curr_ini.texheadertxt(i,:)),'\','\\') '\n']);
            end;
            fprintf(fout,'\n\\newcommand{\\header}[1]{\\vspace*{3mm}\\noindent\\textbf{#1}\\newline\\vspace*{1mm}}\n\\newcommand{\\ruler}{\\vspace*{3mm}\\noindent\\rule{\\columnwidth}{0.4pt}}\n\\newcommand{\\halfruler}{\\vspace*{3mm}\\noindent\\begin{center}\\rule{0.5\\columnwidth}{0.4pt}\\end{center}}\n\\renewcommand{\\vec}{\\mbox{\\rm vec}}\n\n\\makeatletter\n\\def\\mean{\\mathop{\\operator@font mean}}\n\\makeatother\n\n\\begin{document}\n');
         else;
            fprintf(fout,'\n\\clearpage\n\\newline\n');
         end;
         fprintf(fout,['\n\\section*{Structural VAR --- version ' num2str(myvers,'%0.2f') ' \\\\\n']);
         fprintf(fout,['{\\small Copyright \\copyright\\ Anders Warne, 2001-' datestr(now,10) '}}\n\n']);
      else;
         fprintf(fout,'\n');
         fprintf(fout,'*******************************************************************************\n');
         fprintf(fout,'\n');
         fprintf(fout,['               Structural VAR   ---   version ' num2str(myvers,'%0.2f') '\n']);
         fprintf(fout,['               Copyright  Anders Warne, 2001-' datestr(now,10) '\n']);
         fprintf(fout,'\n');
         fprintf(fout,'*******************************************************************************\n');
         fprintf(fout,'\n');
      end;
      %
      if curr_ini.texoutputformat>=2;
         fprintf(fout,'\\begin{tabular}{ll}\n');
         fprintf(fout,['Endogenous variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.evars)));
         fprintf(fout,['Estimation Sample: & ' samplefirst ' -- ' samplelast ' \\\\\n']);
         fprintf(fout,['Initial Sample: & ' initfirst ' -- ' initlast ' \\\\\n']);
         fprintf(fout,['Number of lags: & ' num2str(cm.p) ' for endogenous variables \\\\\n']);
         if cm.q1+cm.q0>0;
            fprintf(fout,['                & ' num2str(cm.pe) ' for exogenous variables \\\\\n']);
         end;
         fprintf(fout,['Number of observations: & ' num2str(size(cm.x0,1)-max(cm.p,cm.pe)) ' \\\\\n']);
         if cm.cointvalue==1;
            fprintf(fout,['Deterministic variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.deter)));
            if strcmp(cm.restvec,'')==0;
               fprintf(fout,[' & %s in cointegration space \\\\\n'],txt2ltxmath(strtrim(cm.restvec)));
            end;
         else;
            fprintf(fout,['Deterministic variables: & %s \\\\\n'],txt2ltxmath(strtrim([cm.deter ' ' cm.restvec])));
         end;
         if strcmp(cm.deterdel,'')==0;
            fprintf(fout,'Deleted variables: & %s \\\\\n',txt2ltxmath(strtrim(cm.deterdel)));
         end;
         if exo==1;
            if cm.q1>0;
               fprintf(fout,['Exogenous I(1) variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.xvars1)));
            end;
            if size(cm.x2,2)-cm.q1>0;
               fprintf(fout,['Exogenous I(0) variables: & %s \\\\\n'],txt2ltxmath(strtrim(cm.xvars0)));
            end;
         end;
         fprintf(fout,'\\end{tabular}\n\n\\ruler\n\n');
         fprintf(fout,'\\subsection*{Bayesian Analysis}\n\n');
      else;
         fprintf(fout,['Endogenous variables:     ' cm.evars '\n']);
         fprintf(fout,['Estimation Sample:        ' samplefirst ' - ' samplelast '\n']);
         fprintf(fout,['Initial Sample:           ' initfirst ' - ' initlast '\n']);
         fprintf(fout,['Number of lags:           ' num2str(cm.p) ' for endogenous variables\n']);
         if cm.q1+cm.q0>0;
            fprintf(fout,['                          ' num2str(cm.pe) ' for exogenous variables\n']);
         end;
         fprintf(fout,['Number of observations:   ' num2str(size(cm.x0,1)-max(cm.p,cm.pe)) '\n']);
         if cm.cointvalue==1;
            fprintf(fout,['Deterministic variables:  ' cm.deter '\n']);
            if strcmp(cm.restvec,'')==0;
               fprintf(fout,['   ' cm.restvec ' in cointegration space\n']);
            end;
         else;
            fprintf(fout,['Deterministic variables:  ' strtrim([cm.deter ' ' cm.restvec]) '\n']);
         end;
         if strcmp(cm.deterdel,'')==0;
            fprintf(fout,'Deleted variables:        %s\n',cm.deterdel);
         end;
         if exo==1;
            if cm.q1>0;
               fprintf(fout,['Exogenous I(1) variables: ' cm.xvars1 '\n']);
            end;
            if size(cm.x2,2)-cm.q1>0;
               fprintf(fout,['Exogenous I(0) variables: ' cm.xvars0 '\n']);
            end;
         end;
         fprintf(fout,'\n*******************************************************************************\n\n');
         fprintf(fout,'BAYESIAN ANALYSIS\n=================\n\n');
      end;
      %
      % write rank probabilities output to file
      %
      if cm.cointvalue==1;
         if curr_ini.texoutputformat>=2;
            prt_str = ['%2.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'];
            fprintf(fout,'\\header{Posterior Cointegration Rank Probabilities} \n\n');
            fprintf(fout,'\\begin{tabular}{rrrr}\n');
%            fprintf(fout,'\\multicolumn{4}{c}{Posterior Cointegration Rank Probabilities} \\\\\n');
            fprintf(fout,'Rank & $P(\\mbox{\\rm Rank}|\\mbox{\\rm Data})$ & $\\log [P(\\mbox{\\rm Data}|\\mbox{\\rm Rank})]$ & $P(\\mbox{\\rm Rank})$ \\\\\n');
            fprintf(fout,prt_str,[(0:1:cm.n)' cm.rankprob frac_marg_like (1/(cm.n+1))*ones(cm.n+1,1)]');
            fprintf(fout,'\\end{tabular}\n\n');
            fprintf(fout,['\\vspace*{2mm}\\noindent\n\\textbf{NOTE:} The calculations are based on the Fractional Marginal Likelihood (FML)\nfor a diffuse prior on the parameters conditional on the rank and equal\nprior probabilities for each rank. The fraction used is minimal, ' num2str(m/cm.T,'%0.2f') ',\nand is the relative part of the sample used to ``train'''' the improper\n(diffuse) prior into a proper distribution.\n\n']);
         else;
            prt_str = ['  %2.0f        %0.' num2str(curr_ini.decimals,'%0.0f') 'f        %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f        %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'];
            fprintf(fout,'Posterior Cointegration Rank Probabilities\n\n Rank    P(Rank|Data)   log[P(Data|Rank)]     P(Rank)\n');
            fprintf(fout,prt_str,[(0:1:cm.n)' cm.rankprob frac_marg_like (1/(cm.n+1))*ones(cm.n+1,1)]');
            fprintf(fout,['\nNOTE: The calculations are based on the Fractional Marginal Likelihood (FML)\n      for a diffuse prior on the parameters conditional on the rank and equal\n      prior probabilities for each rank. The fraction used is minimal, ' num2str(m/cm.T,'%0.2f') ',\n      and is the relative part of the sample used to "train" the improper\n      (diffuse) prior into a proper distribution.\n\n']);
         end;
      end;
      fclose(fout);
   end;
   %
   % we can now remove the wait diaglog
   %
   set(waitf.text,'UserData','done');
   delete(mywait);
   drawnow;
   %
   % Start up the Bayesian analysis GUI
   %
   bayesdlg = bayesgui('init',cm);
   drawnow;
   controls = get(bayesdlg,'UserData');
   delete(bayesdlg);
   drawnow;
   %
   % closing down the show
   %
   setlogtext(strvcat('Bayesian analysis completed!'),maingui);
   if curr_ini.noprint==0;
      %
      % Close the outputfile
      %
      fout = fopen(cm.outputfile,'at');
      if curr_ini.texoutputformat>=2;
         fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
         fprintf(fout,['\\header{Output created on: ' datestr(now,0) '}\n']);
      else;
         fprintf(fout,'*******************************************************************************\n\n');
         fprintf(fout,['Output created on: ' datestr(now,0) '\n']);
      end;
      fclose(fout);
   end;
end;

%
% end of varrun.m
%